@@ -1,24 +1,3 @@
-/*
-Copyright (C) 1999-2006 Id Software, Inc. and contributors.
-For a list of contributors, see the accompanying CONTRIBUTORS file.
-
-This file is part of GtkRadiant.
-
-GtkRadiant is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-GtkRadiant is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GtkRadiant; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-*/
-
 /**
  * \defgroup eclass Entity class manager
  * \file ieclass.h
@@ -30,7 +9,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include "ModResource.h"
 
 #include "imodule.h"
-#include "math/Vector3.h"
+#include "ifilesystem.h"
+#include "math/Vector4.h"
 
 #include <vector>
 #include <list>
@@ -138,7 +118,7 @@ typedef std::shared_ptr<const IEntityClass> IEntityClassConstPtr;
  *
  * \ingroup eclass
  */
-class IEntityClass : 
+class IEntityClass :
     public ModResource
 {
 public:
@@ -148,11 +128,14 @@ public:
     virtual sigc::signal<void>& changedSignal() = 0;
 
     /// Get the name of this entity class
-    virtual std::string getName() const = 0;
+    virtual const std::string& getName() const = 0;
 
     /// Get the parent entity class or NULL if there is no parent
     virtual const IEntityClass* getParent() const = 0;
 
+    /// Get the UI visibility of this entity class
+    virtual vfs::Visibility getVisibility() const = 0;
+
     /// Query whether this entity class represents a light.
     virtual bool isLight() const = 0;
 
@@ -174,49 +157,30 @@ public:
     /* ENTITY CLASS COLOURS */
 
     /// Return the display colour of this entity class
-    virtual const Vector3& getColour() const = 0;
+    virtual const Vector4& getColour() const = 0;
 
     // Overrides the colour defined in the .def files
-    virtual void setColour(const Vector3& colour) = 0;
-
-    /// Get the shader used for rendering this entity class in wireframe mode.
-    virtual const std::string& getWireShader() const = 0;
-
-    /// Get the shader used for rendering this entity class in filled mode.
-    virtual const std::string& getFillShader() const = 0;
-
+    virtual void setColour(const Vector4& colour) = 0;
 
     /* ENTITY CLASS ATTRIBUTES */
 
     /**
-     * Return a single named EntityClassAttribute from this EntityClass.
+     * @brief Get the value of a specified attribute.
      *
-     * \param name
-     * The name of the EntityClassAttribute to find, interpreted case-insensitively.
-     *
-     * \param includeInherited
-     * true if attributes inherited from parent entity classes should be
-     * considered, false otherwise.
-     *
-     * \return
-     * A reference to the named EntityClassAttribute. If the named attribute is
-     * not found, an empty EntityClassAttribute is returned.
+     * @return std::string containing the attribute value, or an empty string if the attribute was
+     * not found.
      */
-    virtual EntityClassAttribute& getAttribute(const std::string& name, 
-        bool includeInherited = true) = 0;
-
-    /// Get a const EntityClassAttribute reference by name
-    virtual const EntityClassAttribute& getAttribute(const std::string& name,
-                 bool includeInherited = true) const = 0;
+    virtual std::string getAttributeValue(const std::string& name,
+                                          bool includeInherited = true) const = 0;
 
     // Returns the attribute type string for the given name.
     // This method will walk up the inheritance hierarchy until it encounters a type definition.
     // If no type is found, an empty string will be returned.
-    virtual const std::string& getAttributeType(const std::string& name) const = 0;
+    virtual std::string getAttributeType(const std::string& name) const = 0;
 
     // Returns the attribute description string for the given name.
     // This method will walk up the inheritance hierarchy until it encounters a non-empty description.
-    virtual const std::string& getAttributeDescription(const std::string& name) const = 0;
+    virtual std::string getAttributeDescription(const std::string& name) const = 0;
 
     /**
      * Function that will be invoked by forEachAttribute.
@@ -238,8 +202,7 @@ public:
      * true if editor keys (those which start with "editor_") should be passed
      * to the visitor, false if they should be skipped.
      */
-    virtual void forEachAttribute(AttributeVisitor visitor,
-                                  bool editorKeys = false) const = 0;
+    virtual void forEachAttribute(AttributeVisitor visitor, bool editorKeys = false) const = 0;
 
     /* MODEL AND SKIN */
 
@@ -340,13 +303,13 @@ class IEntityClassManager :
 {
 public:
     /// Signal emitted when starting to parse DEFs
-    virtual sigc::signal<void> defsLoadingSignal() const = 0;
+    virtual sigc::signal<void>& defsLoadingSignal() = 0;
 
     /// Signal emitted when all DEFs have been loaded (after module initialisation)
-    virtual sigc::signal<void> defsLoadedSignal() const = 0;
+    virtual sigc::signal<void>& defsLoadedSignal() = 0;
 
     /// Signal emitted when all DEFs are reloaded
-    virtual sigc::signal<void> defsReloadedSignal() const = 0;
+    virtual sigc::signal<void>& defsReloadedSignal() = 0;
 
     /**
      * Return the IEntityClass corresponding to the given name, creating it if
