@@ -16,9 +16,11 @@ typedef std::shared_ptr<IEntityClass> IEntityClassPtr;
 typedef std::shared_ptr<const IEntityClass> IEntityClassConstPtr;
 
 // Observes a single entity key value and gets notified on change
-class KeyObserver
+class KeyObserver: public sigc::trackable
 {
 public:
+    using Ptr = std::shared_ptr<KeyObserver>;
+
     virtual ~KeyObserver() {}
 
     /**
@@ -28,24 +30,40 @@ public:
     virtual void onKeyValueChanged(const std::string& newValue) = 0;
 };
 
-class EntityKeyValue :
-    public NameObserver
+/**
+ * @brief Object representing a single keyvalue (spawnarg) on an entity.
+ *
+ * This class exists so that each spawnarg can have its own independent set of
+ * KeyObservers responding to changes in its value. For most purposes it is
+ * simpler to use Entity::Observer::onKeyChange, Entity::setKeyValue and
+ * Entity::getKeyValue to interact with key values.
+ */
+class EntityKeyValue: public NameObserver
 {
 public:
     virtual ~EntityKeyValue() {}
-    /** greebo: Retrieves the actual value of this key
-     */
+
+    /// Retrieves the actual value of this key
     virtual const std::string& get() const = 0;
 
-    /** greebo: Sets the value of this key
-     */
+    /// Sets the value of this key
     virtual void assign(const std::string& other) = 0;
 
-    /** greebo: Attaches/detaches a callback to get notified about
-     *          the key change.
-     */
+    /// Attaches a callback to get notified about the key change.
     virtual void attach(KeyObserver& observer) = 0;
-    virtual void detach(KeyObserver& observer) = 0;
+
+    /**
+     * @brief Detach the given observer from this key value.
+     *
+     * @param observer
+     * Observer to detach. No action will be taken if this observer is not
+     * already attached.
+     *
+     * @param sendEmptyValue
+     * If true (the default), the observer will be invoked with an empty value
+     * before being detached. If false, no final value will be sent.
+     */
+    virtual void detach(KeyObserver& observer, bool sendEmptyValue = true) = 0;
 };
 typedef std::shared_ptr<EntityKeyValue> EntityKeyValuePtr;
 
@@ -267,6 +285,9 @@ public:
     virtual void forEachAttachment(AttachmentFunc func) const = 0;
 };
 
+/// Callback for an entity key value change
+using KeyObserverFunc = sigc::slot<void(const std::string&)>;
+
 /**
  * \brief Interface for a node which represents an entity.
  *
@@ -285,12 +306,34 @@ public:
     /// Get a modifiable reference to the contained Entity
     virtual Entity& getEntity() = 0;
 
+    /**
+     * @brief Observe key value changes using a callback function.
+     *
+     * This method provides a simpler interface for observing key value changes
+     * via the use of a callback function, rather than requiring a full
+     * KeyObserver object to be constructed and maintained by the calling code.
+     *
+     * @param key
+     * The key to observe.
+     *
+     * @param func
+     * Function to call when the key value changes.
+     */
+    virtual void observeKey(const std::string& key, KeyObserverFunc func) = 0;
+
     /**
      * greebo: Tells the entity to reload the child model. This usually
      *         includes removal of the child model node and triggering
      *         a "skin changed" event.
      */
     virtual void refreshModel() = 0;
+
+    /**
+     * Invokes the given function object for each attached entity.
+     * At this point attachment entities are not accessible through the node's children,
+     * they have to be accessed through this method instead.
+     */
+    virtual void foreachAttachment(const std::function<void(const IEntityNodePtr&)>& functor) = 0;
 };
 typedef std::shared_ptr<IEntityNode> IEntityNodePtr;
 
@@ -326,7 +369,7 @@ public:
     // Iterates over each selected entity, invoking the given functor
     virtual void foreachEntity(const std::function<void(Entity*)>& functor) = 0;
 
-    // Returns the key value shared by all entities in this set, or an empty string 
+    // Returns the key value shared by all entities in this set, or an empty string
     // if there is no such value.
     virtual std::string getSharedKeyValue(const std::string& key, bool includeInherited) = 0;
 };
@@ -382,7 +425,7 @@ public:
      * Returns the Target with the given name.
      * This never returns NULL, an ITargetableObject is created if it doesn't exist yet.
      */
-    virtual ITargetableObjectPtr getTarget(const std::string name) = 0;
+    virtual ITargetableObjectPtr getTarget(const std::string& name) = 0;
 
     /**
      * greebo: Associates the named Target with the given scene::INode.
@@ -390,6 +433,12 @@ public:
      */
     virtual void associateTarget(const std::string& name, const scene::INode& node) = 0;
 
+    // Will be called by a TargetableNode to notify about visibility changes
+    virtual void onTargetVisibilityChanged(const std::string& name, const scene::INode& node) = 0;
+
+    // Will be called by a TargetableNode to notify about a position change
+    virtual void onTargetPositionChanged(const std::string& name, const scene::INode& node) = 0;
+
     /**
      * greebo: Disassociates the Target from the given name. The node
      * must also be passed to allow the manager to check the request.
