@@ -5,9 +5,9 @@
 #include "math/AABB.h"
 
 /**
- * LightNodes derive from this class. 
+ * LightNodes derive from this class.
  * It's mainly used to determine the selectable part
- * of the light, which is usually the small "diamond" in 
+ * of the light, which is usually the small "diamond" in
  * the center.
  */
 class ILightNode
@@ -15,9 +15,10 @@ class ILightNode
 public:
     virtual ~ILightNode() {}
 
-    /** 
-	 * greebo: Get the AABB of the Light "Diamond" representation.
-     */
+    /// Return the RendererLight instance for this light node
+    virtual const RendererLight& getRendererLight() const = 0;
+
+	/// Get the AABB of the Light "Diamond" representation.
     virtual AABB getSelectAABB() const = 0;
 };
 typedef std::shared_ptr<ILightNode> ILightNodePtr;
