@@ -1,10 +1,19 @@
 #pragma once
 
 #include "imodule.h"
+#include "ivolumetest.h"
+#include "irenderview.h"
+#include "iwindingrenderer.h"
+#include "igeometryrenderer.h"
+#include "isurfacerenderer.h"
 #include <functional>
+#include <vector>
 
 #include "math/Vector3.h"
+#include "math/Vector4.h"
+#include "render/Colour4.h"
 #include "math/AABB.h"
+#include "render/MeshVertex.h"
 
 #include "ishaderlayer.h"
 #include <sigc++/signal.h>
@@ -117,6 +126,8 @@ typedef BasicVector3<double> Vector3;
 class Shader;
 typedef std::shared_ptr<Shader> ShaderPtr;
 
+struct RenderableGeometry;
+
 /**
  * A RenderEntity represents a map entity as seen by the renderer.
  * It provides up to 12 numbered parameters to the renderer:
@@ -129,6 +140,8 @@ typedef std::shared_ptr<Shader> ShaderPtr;
 class IRenderEntity
 {
 public:
+    // Returns the name of this entity (mainly for debugging purposes)
+    virtual std::string getEntityName() const = 0;
 
 	/**
 	 * Get the value of this entity's shader parm with the given index.
@@ -144,6 +157,47 @@ public:
 	 * Returns the wireframe shader for this entity - child primitives need this for rendering.
 	 */
 	virtual const ShaderPtr& getWireShader() const = 0;
+
+    /**
+     * Returns the shader that is used to render coloured primitives like lines and points,
+     * using the colour of the entity as defined by its entityDef, light colour, etc.
+     * This shader will be applicable to both camera and ortho views, it can be used for
+     * visualisations such as target lines and light boxes.
+     */
+    virtual const ShaderPtr& getColourShader() const = 0;
+
+    /**
+     * Returns the colour of this entity, that is used to display its
+     * wireframe representation.
+     */
+    virtual Vector4 getEntityColour() const = 0;
+
+    /**
+     * Associates the given object with this entity and the given shader.
+     * It will be processed during the following lighting mode rendering passes.
+     */
+    virtual void addRenderable(const render::IRenderableObject::Ptr& object, Shader* shader) = 0;
+
+    /**
+     * Removes the object from this entity.
+     */
+    virtual void removeRenderable(const render::IRenderableObject::Ptr& object) = 0;
+
+    using ObjectVisitFunction = std::function<void(const render::IRenderableObject::Ptr&, Shader*)>;
+
+    /**
+     * Enumerate all entity object, unconditionally.
+     */
+    virtual void foreachRenderable(const ObjectVisitFunction& functor) = 0;
+
+    /**
+     * Enumerate all entity object (partially) intersecting with the given bounds.
+     * The bounds are specified in world coordinates.
+     */
+    virtual void foreachRenderableTouchingBounds(const AABB& bounds, const ObjectVisitFunction& functor) = 0;
+
+    // Returns true if this entity produces shadows when lit (i.e.returns false when the entity has "noshadows" set to 1)
+    virtual bool isShadowCasting() const = 0;
 };
 typedef std::shared_ptr<IRenderEntity> IRenderEntityPtr;
 typedef std::weak_ptr<IRenderEntity> IRenderEntityWeakPtr;
@@ -208,6 +262,9 @@ public:
      * pyramid (the same as worldOrigin()).
      */
 	virtual Vector3 getLightOrigin() const = 0;
+
+    // Whether this light is allowed to cast shadows
+    virtual bool isShadowCasting() const = 0;
 };
 typedef std::shared_ptr<RendererLight> RendererLightPtr;
 
@@ -218,142 +275,19 @@ inline std::ostream& operator<< (std::ostream& os, const RendererLight& l)
               << ", lightAABB=" << l.lightAABB() << ")";
 }
 
-/**
- * \brief Interface for an object which can test its intersection with a
- * RendererLight.
- *
- * Objects which implement this interface define a intersectsLight() function
- * which determines whether the given light intersects the object.
- */
-class LitObject
-{
-public:
-    virtual ~LitObject() {}
-
-    /// Test if the given light intersects the LitObject
-    virtual bool intersectsLight(const RendererLight& light) const = 0;
-};
-typedef std::shared_ptr<LitObject> LitObjectPtr;
-
 class Renderable;
-typedef std::function<void(const Renderable&)> RenderableCallback;
+typedef std::function<void(Renderable&)> RenderableCallback;
 
 typedef std::function<void(const RendererLight&)> RendererLightCallback;
 
-/**
- * \brief
- * Simple container of light sources
- *
- * This is a storage class used to represent all light sources which fall upon
- * a particular object. It is passed to the RenderSystem at render time to
- * provide the list of lights which intersect the Renderable being submitted.
- */
-class LightSources
-{
-public:
-
-    /// Invoke a callback on all contained lights.
-    virtual void forEachLight(const RendererLightCallback& callback) const = 0;
-};
-
-/// Debug stream insertion for LightSources
-inline std::ostream& operator<< (std::ostream& s, const LightSources& ls)
-{
-    s << "LightSources(";
-
-    // Insert comma-separated list of RendererLights
-    bool addComma = false;
-    ls.forEachLight(
-        [&](const RendererLight& l)
-        {
-            if (addComma)
-                s << ", ";
-            s << l;
-            addComma = true;
-        }
-    );
-
-    return s << ")";
-}
-
-/// Debug stream insertion for possibly null LightSources pointer
-inline std::ostream& operator<< (std::ostream& s, const LightSources* ls)
-{
-    if (ls)
-        return s << *ls;
-    else
-        return s << "[no lightsources]";
-}
-
-const int c_attr_TexCoord0 = 1;
-const int c_attr_Tangent = 3;
-const int c_attr_Binormal = 4;
-
-/**
- * \brief
- * Data object passed to the backend OpenGLRenderable::render() method
- * containing information about the render pass which may be of use to
- * renderable objects, including the render flags and various
- * matrices/coordinates.
- */
-class RenderInfo
-{
-    // Render flags
-    RenderStateFlags _flags;
-
-    // Viewer location in 3D space
-    Vector3 _viewerLocation;
-
-    // Cube map mode
-    IShaderLayer::CubeMapMode _cubeMapMode;
-
-public:
-
-    /// Default constructor
-    RenderInfo(RenderStateFlags flags = RENDER_DEFAULT,
-               const Vector3& viewer = Vector3(0, 0, 0),
-               IShaderLayer::CubeMapMode cubeMode = IShaderLayer::CUBE_MAP_NONE)
-    : _flags(flags),
-      _viewerLocation(viewer),
-      _cubeMapMode(cubeMode)
-    { }
-
-    /// Check if a flag is set
-    bool checkFlag(unsigned flag) const
-    {
-        return (_flags & flag) != 0;
-    }
-
-    /// Get the entire flag bitfield.
-    RenderStateFlags getFlags() const
-    {
-        return _flags;
-    }
-
-    /// Get the viewer location.
-    const Vector3& getViewerLocation() const
-    {
-        return _viewerLocation;
-    }
-
-    /// Get the cube map mode.
-    IShaderLayer::CubeMapMode getCubeMapMode() const
-    {
-        return _cubeMapMode;
-    }
-};
-
 /**
  * \brief
  * Interface for objects which can render themselves in OpenGL.
  *
- * This interface is used by the render backend, after renderable objects have
+ * This interface is used for highlight rendering, after renderable objects have
  * first been submitted using the Renderable interface. The backend render()
  * function should contain the OpenGL calls necessary to submit vertex, normal
  * and texture-coordinate data.
- *
- * No GL state changes should occur in render(), other than those specifically
- * allowed by the render flags.
  */
 class OpenGLRenderable
 {
@@ -364,7 +298,7 @@ public:
      * \brief
      * Submit OpenGL render calls.
      */
-    virtual void render(const RenderInfo& info) const = 0;
+    virtual void render() const = 0;
 };
 
 class Matrix4;
@@ -384,7 +318,10 @@ typedef std::shared_ptr<Material> MaterialPtr;
  * which use it -- the actual rendering is performed by traversing a list of
  * Shaders and rendering the geometry attached to each one.
  */
-class Shader
+class Shader :
+    public render::IWindingRenderer,
+    public render::IGeometryRenderer,
+    public render::ISurfaceRenderer
 {
 public:
 	// Observer interface to get notified on (un-)realisation
@@ -410,18 +347,9 @@ public:
 	 *
 	 * \param modelview
 	 * The modelview transform for this object.
-	 *
-	 * \param lights
-     * Optional LightSources containing all of the lights which illuminate this
-     * object.
-     *
-     * \param entity
-     * Optional IRenderEntity exposing entity-related render parameters.
 	 */
 	virtual void addRenderable(const OpenGLRenderable& renderable,
-							   const Matrix4& modelview,
-							   const LightSources* lights = nullptr,
-                               const IRenderEntity* entity = nullptr) = 0;
+							   const Matrix4& modelview) = 0;
 
     /**
      * \brief
@@ -465,8 +393,148 @@ public:
  */
 typedef std::shared_ptr<Shader> ShaderPtr;
 
+// Defines a piece of text that should be rendered in the scene
+class IRenderableText
+{
+public:
+    // The position in world space where the text needs to be rendered
+    virtual const Vector3& getWorldPosition() = 0;
+
+    // The text to be rendered. Returning an empty text will skip this renderable.
+    virtual const std::string& getText() = 0;
+
+    // The colour of the text
+    virtual const Vector4& getColour() = 0;
+};
+
+// An ITextRenderer is able to draw one or more IRenderableText
+// instances to the scene
+class ITextRenderer
+{
+public:
+    using Ptr = std::shared_ptr<ITextRenderer>;
+
+    using Slot = std::uint64_t;
+    static constexpr Slot InvalidSlot = std::numeric_limits<Slot>::max();
+
+    // Registers the given text instance for rendering
+    virtual Slot addText(IRenderableText& text) = 0;
+
+    // Removes the text instance in the given Slot
+    // The slot handle is invalidated by this operation and should be 
+    // discarded by the client, by setting it to InvalidSlot
+    virtual void removeText(Slot slot) = 0;
+};
+
 const char* const MODULE_RENDERSYSTEM("ShaderCache");
 
+// Render view type enumeration, is used to select (or leave out)
+// Shader objects during rendering
+enum class RenderViewType
+{
+    Camera      = 1 << 0,
+    OrthoView   = 1 << 1,
+};
+
+enum class BuiltInShaderType
+{
+    // A medium-sized point
+    Point,
+
+    // A bigger version of "Point"
+    BigPoint,
+
+    // Lines connecting the patch control points
+    PatchLattice,
+
+    // Line shader drawing above regular things
+    WireframeOverlay,
+
+    // Fill shader drawing above regular things
+    FlatshadeOverlay,
+
+    // Line shader used for drawing AAS area bounds
+    AasAreaBounds,
+
+    // Shader used for the boxes marking missing models
+    MissingModel,
+
+    // The polygon showing the clip plane intersecting brushes
+    BrushClipPlane,
+
+    // Line shader connecting the leak point trace
+    PointTraceLines,
+
+    // This is the shader drawing a coloured overlay
+    // over faces/polys. Its colour is configurable,
+    // and it has depth test activated (camera).
+    ColouredPolygonOverlay,
+
+    // This is the shader drawing a solid line to outline
+    // a selected item. The first pass has its depth test
+    // activated using GL_LESS, whereas the second pass
+    // draws the hidden lines in stippled appearance
+    // with its depth test using GL_GREATER (camera).
+    HighlightedPolygonOutline,
+
+    // Coloured line stipple overlay of selected items (ortho)
+    WireframeSelectionOverlay,
+
+    // Coloured line stipple overlay of selected grouped items (ortho)
+    WireframeSelectionOverlayOfGroups,
+
+    // Line shader used for drawing the three axes at entity origins
+    Pivot,
+
+    // Coloured merge action highlights (camera)
+    CameraMergeActionOverlayAdd,
+    CameraMergeActionOverlayRemove,
+    CameraMergeActionOverlayChange,
+    CameraMergeActionOverlayConflict,
+
+    // Coloured merge action highlights (ortho)
+    OrthoMergeActionOverlayAdd,
+    OrthoMergeActionOverlayRemove,
+    OrthoMergeActionOverlayChange,
+    OrthoMergeActionOverlayConflict,
+};
+
+// Available types of colour shaders. These areused 
+// to draw map items in the ortho view, or to render
+// connection lines, light volumes in both camera and ortho.
+enum class ColourShaderType
+{
+    // Line shader for camera views only
+    CameraOutline,
+
+    // Fill shader, non-transparent
+    CameraSolid,
+
+    // Fill shader with transparency
+    CameraTranslucent,
+
+    // Solid lines used for ortho view rendering
+    OrthoviewSolid,
+
+    // Items drawn in both camera and ortho views
+    CameraAndOrthoview,
+
+    // Outline shader visible in camera and ortho views
+    CameraAndOrthoViewOutline,
+};
+
+class IRenderResult
+{
+public:
+    using Ptr = std::shared_ptr<IRenderResult>;
+
+    virtual ~IRenderResult() {}
+
+    virtual std::string toString() = 0;
+};
+
+constexpr const char* const RKEY_ENABLE_SHADOW_MAPPING = "user/ui/renderSystem/enableShadowMapping";
+
 /**
  * \brief
  * The main interface for the backend renderer.
@@ -481,8 +549,6 @@ public:
      * Capture the given shader, increasing its reference count and
 	 * returning a pointer to the Shader object.
      *
-     * The object must be freed after use by calling release().
-	 *
 	 * @param name
 	 * The name of the shader to capture.
 	 *
@@ -492,6 +558,50 @@ public:
 
 	virtual ShaderPtr capture(const std::string& name) = 0;
 
+    /**
+     * Retrieves an ITextRenderer instance with the given font style and size.
+     * This renderer will accept IRenderableText instances which define
+     * the actual text, colour and position.
+     */
+    virtual ITextRenderer::Ptr captureTextRenderer(IGLFont::Style style, std::size_t size) = 0;
+
+    /**
+     * Acquire one of DarkRadiant's built-in shaders, used for drawing
+     * things like connectors, lines, points and overlays.
+     */
+    virtual ShaderPtr capture(BuiltInShaderType type) = 0;
+
+    /**
+     * Capture a colour shader, there are various types available
+     * for rendering in ortho views, camera or both.
+     */
+    virtual ShaderPtr capture(ColourShaderType type, const Colour4& colour) = 0;
+
+    /**
+     * Register the given entity to be considered during rendering.
+     * If this entity is a light it will be automatically recognised 
+     * and inserted into the set of lights.
+     */
+    virtual void addEntity(const IRenderEntityPtr& renderEntity) = 0;
+
+    /**
+     * Detaches this entity from this rendersystem, it won't be 
+     * affected by any rendering after this call.
+     */
+    virtual void removeEntity(const IRenderEntityPtr& renderEntity) = 0;
+
+    /**
+     * Enumerates all known render entities in this system, invoking
+     * the functor for each of them.
+     */
+    virtual void foreachEntity(const std::function<void(const IRenderEntityPtr&)>& functor) = 0;
+
+    /**
+     * Enumerates all known render lights in this system, invoking
+     * the functor for each of them
+     */
+    virtual void foreachLight(const std::function<void(const RendererLightPtr&)>& functor) = 0;
+
     /**
      * \brief
      * Main render method.
@@ -505,19 +615,30 @@ public:
      * render flag which is 0 in this mask will not be enabled during rendering,
      * even if the particular shader requests it.
      *
-     * \param modelview
-     * The modelview transformation matrix to apply before rendering.
-     *
-     * \param projection
-     * The view projection matrix to apply before rendering.
-     *
-     * \param viewer
-     * Location of the viewer in world space.
+     * * \param view
+     * The view used to setup the projection, modelview and lighting calculations.
      */
-    virtual void render(RenderStateFlags globalFlagsMask,
-                        const Matrix4& modelview,
-                        const Matrix4& projection,
-                        const Vector3& viewer) = 0;
+    virtual IRenderResult::Ptr renderFullBrightScene(RenderViewType renderViewType,
+                        RenderStateFlags globalFlagsMask,
+                        const render::IRenderView& view) = 0;
+
+    virtual void startFrame() = 0;
+    virtual void endFrame() = 0;
+
+    /**
+     * Render the scene based on the light-entity interactions.
+     * All the active lights and entities must have added themselves
+     * to this rendersystem at this point, using addEntity().
+     * 
+     * \param globalFlagsMask
+     * The mask of render flags which are permitted during this render pass. Any
+     * render flag which is 0 in this mask will not be enabled during rendering,
+     * even if the particular shader requests it.
+     * 
+     * \returns A result object which can be used to display a statistics summary.
+     */
+    virtual IRenderResult::Ptr renderLitScene(RenderStateFlags globalFlagsMask, 
+        const render::IRenderView& view) = 0;
 
     virtual void realise() = 0;
     virtual void unrealise() = 0;
@@ -550,8 +671,8 @@ public:
     /// Set the shader program to use.
     virtual void setShaderProgram(ShaderProgram prog) = 0;
 
-    virtual void attachRenderable(const Renderable& renderable) = 0;
-    virtual void detachRenderable(const Renderable& renderable) = 0;
+    virtual void attachRenderable(Renderable& renderable) = 0;
+    virtual void detachRenderable(Renderable& renderable) = 0;
     virtual void forEachRenderable(const RenderableCallback& callback) const = 0;
 
   	// Initialises the OpenGL extensions
@@ -563,6 +684,9 @@ public:
     // Sets the flag whether shader programs are available.
     virtual void setShaderProgramsAvailable(bool available) = 0;
 
+    // Activates or deactivates the merge render mode
+    virtual void setMergeModeEnabled(bool enabled) = 0;
+
 	// Subscription to get notified as soon as the openGL extensions have been initialised
 	virtual sigc::signal<void> signal_extensionsInitialised() = 0;
 };
