@@ -235,6 +235,55 @@ inline bool Node_hasSelectedChildNodes(const scene::INodePtr& node)
 namespace scene
 {
 
+inline void setNodeHidden(const INodePtr& node, bool hide)
+{
+    if (!node->supportsStateFlag(Node::eHidden))
+    {
+        return;
+    }
+
+    if (hide)
+    {
+        node->enable(Node::eHidden);
+    }
+    else
+    {
+        node->disable(Node::eHidden);
+    }
+}
+
+inline void showNode(const INodePtr& node)
+{
+    setNodeHidden(node, false);
+}
+
+inline void showSubgraph(const INodePtr& node)
+{
+    showNode(node);
+
+    node->foreachNode([&](const INodePtr& child)
+    {
+        showNode(child);
+        return true;
+    });
+}
+
+inline void hideNode(const INodePtr& node)
+{
+    setNodeHidden(node, true);
+}
+
+inline void hideSubgraph(const INodePtr& node)
+{
+    hideNode(node);
+
+    node->foreachNode([&](const INodePtr& child)
+    {
+        hideNode(child);
+        return true;
+    });
+}
+
 /**
  * greebo: This walker removes all encountered child nodes without
  * traversing each node's children. This deselects all removed nodes as well.
@@ -283,13 +332,12 @@ inline void assignVisibilityFlagsFromNode(INode& target, const INode& source)
     }
 }
 
-inline std::pair<Vector3, Vector3> getOriginAndAnglesToLookAtNode(const scene::INode& node)
+inline std::pair<Vector3, Vector3> getOriginAndAnglesToLookAtBounds(const AABB& aabb)
 {
-    const AABB& aabb = node.worldAABB();
     Vector3 origin(aabb.origin);
 
     // Move the camera a bit off the AABB origin
-    origin += Vector3(aabb.extents.getLength() * 5, 0, aabb.extents.getLength() * 5);
+    origin += Vector3(aabb.extents.getLength() * 3, 0, aabb.extents.getLength() * 3);
 
     // Rotate the camera a bit towards the "ground"
     Vector3 angles(0, 0, 0);
@@ -299,4 +347,9 @@ inline std::pair<Vector3, Vector3> getOriginAndAnglesToLookAtNode(const scene::I
     return std::make_pair(origin, angles);
 }
 
+inline std::pair<Vector3, Vector3> getOriginAndAnglesToLookAtNode(const scene::INode& node)
+{
+    return getOriginAndAnglesToLookAtBounds(node.worldAABB());
+}
+
 } // namespace scene
