@@ -28,6 +28,16 @@ class EntityClassChooser final :
     public DialogBase,
     private XmlResourceBasedWidget
 {
+public:
+    // Enumeration of possible modes of this dialog
+    // Influences the dialog title and button labels
+    enum class Purpose
+    {
+        AddEntity,
+        ConvertEntity,
+        SelectClassname,
+    };
+
 private:
     ResourceTreeView::Columns _columns;
     ResourceTreeView* _treeView;
@@ -45,7 +55,7 @@ private:
     sigc::connection _defsReloaded;
 
 private:
-    EntityClassChooser();
+    EntityClassChooser(Purpose purpose);
     ~EntityClassChooser();
 
     void loadEntityClasses();
@@ -81,12 +91,15 @@ public:
     /**
      * \brief Construct and show the dialog to choose an entity class,
      * returning the result.
+     * 
+     * \param purpose
+     * The scenario this dialog has been requested for
      *
      * \param preselectEclass
      * Optional initial class to locate and highlight in the tree after the
      * dialog is shown.
      */
-    static std::string chooseEntityClass(const std::string& preselectEclass = {});
+    static std::string ChooseEntityClass(Purpose purpose, const std::string& preselectEclass = {});
 };
 
 } // namespace
