@@ -7,6 +7,7 @@
 #include "inode.h"
 #include "ieclass.h"
 #include "ientity.h"
+#include "imap.h"
 
 namespace ui
 {
@@ -15,6 +16,9 @@ class AnimationPreview :
 	public wxutil::RenderPreview
 {
 private:
+    // The scene root
+    scene::IMapRootNodePtr _root;
+
 	// Current MD5 model node to display
 	scene::INodePtr _model;
 
@@ -42,17 +46,20 @@ public:
 		return _anim;
 	}
 
-	AABB getSceneBounds();
+    AABB getSceneBounds() override;
 
 protected:
 	// Creates parent entity etc.
-	void setupSceneGraph();
+    void setupSceneGraph() override;
 
-	bool onPreRender();
+    bool onPreRender() override;
+    std::string getInfoText() override;
 
 	void clearModel();
 
-	RenderStateFlags getRenderFlagsFill();
+    RenderStateFlags getRenderFlagsFill() override;
+
+    void onModelRotationChanged() override;
 };
 typedef std::shared_ptr<AnimationPreview> AnimationPreviewPtr;
 
