@@ -159,7 +159,7 @@ void EntityInspector::construct()
 
     // Stimulate initial redraw to get the correct status
     requestIdleCallback();
-    
+
     _defsReloadedHandler = GlobalEntityClassManager().defsReloadedSignal().connect(
         sigc::mem_fun(this, &EntityInspector::onDefsReloaded)
     );
@@ -363,6 +363,8 @@ void EntityInspector::applyMergeActionStyle(const std::string& key, wxDataViewIt
         case scene::merge::ActionType::RemoveKeyValue:
             wxutil::TreeViewItemStyle::ApplyKeyValueRemovedStyle(style);
             break;
+        default:
+            break;
         }
     }
 
@@ -669,7 +671,7 @@ std::string EntityInspector::getSelectedKey()
 
     wxutil::TreeModel::Row row(item, *_keyValueTreeView->GetModel());
 
-    wxDataViewIconText iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
+    auto iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
     return iconAndName.GetText().ToStdString();
 }
 
@@ -730,9 +732,6 @@ void EntityInspector::updateGUIElements()
     {
         _editorFrame->Enable(entityCanBeUpdated);
         _showHelpColumnCheckbox->Enable(true);
-        _keyEntry->Enable(entityCanBeUpdated);
-        _valEntry->Enable(entityCanBeUpdated);
-        _setButton->Enable(entityCanBeUpdated);
         _booleanColumn->GetRenderer()->SetMode(entityCanBeUpdated ? wxDATAVIEW_CELL_ACTIVATABLE : wxDATAVIEW_CELL_INERT);
 
         if (!entityCanBeUpdated)
@@ -749,6 +748,20 @@ void EntityInspector::updateGUIElements()
         _showInheritedCheckbox->Enable(false);
         _showHelpColumnCheckbox->Enable(false);
     }
+
+    updateEntryBoxSensitivity();
+}
+
+void EntityInspector::updateEntryBoxSensitivity()
+{
+    auto entityCanBeUpdated = canUpdateEntity() && !_entitySelection->empty();
+    auto selectedKey = getSelectedKey();
+
+    auto classNameSelected = getSelectedKey() == "classname";
+
+    _keyEntry->Enable(entityCanBeUpdated && !classNameSelected);
+    _valEntry->Enable(entityCanBeUpdated && !classNameSelected);
+    _setButton->Enable(entityCanBeUpdated && !classNameSelected);
 }
 
 void EntityInspector::updatePrimitiveNumber()
@@ -800,7 +813,7 @@ void EntityInspector::updatePrimitiveNumber()
             rWarning() << ex.what() << std::endl;
         }
     }
-    
+
     _primitiveNumLabel->SetLabelText("");
 }
 
@@ -813,7 +826,7 @@ void EntityInspector::onIdle()
         // Fire the selection update. This will invoke onKeyAdded/onKeyChanged etc.
         // on ourselves for every spawnarg that should be listed or removed
         _entitySelection->update();
-        
+
         // After selection rescan, trigger an update of the key types
         // of all listed key/value pairs. Not all information is fully available
         // during the above update
@@ -865,14 +878,14 @@ void EntityInspector::onIdle()
 
         _showInheritedCheckbox->Enable(canShowInheritedKeys);
 
+        // We might be switching from one single selected entity to another
+        // Avoid the old set of inherited key values from sticking around
+        removeClassProperties();
+
         if (canShowInheritedKeys && _showInheritedCheckbox->IsChecked())
         {
             addClassProperties();
         }
-        else
-        {
-            removeClassProperties();
-        }
     }
 
     if (_helpTextNeedsUpdate)
@@ -1171,7 +1184,7 @@ void EntityInspector::_onRejectMergeAction()
         wxutil::TreeModel::Row row(item, *_kvStore);
 
         auto key = row[_columns.name].getString().ToStdString();
-        
+
         auto conflict = _conflictActions.find(key);
 
         if (conflict != _conflictActions.end())
@@ -1182,7 +1195,7 @@ void EntityInspector::_onRejectMergeAction()
         }
 
         auto action = _mergeActions.find(key);
-        
+
         if (action != _mergeActions.end())
         {
             action->second->deactivate();
@@ -1386,7 +1399,7 @@ void EntityInspector::updateHelpText(const wxutil::TreeModel::Row& row)
 
     // Check the entityclass (which will return blank if not found)
     auto eclass = _entitySelection->getSingleSharedEntityClass();
-    
+
     if (!eclass)
     {
         setHelpText("");
@@ -1481,6 +1494,8 @@ void EntityInspector::_onTreeViewSelectionChanged(wxDataViewEvent& ev)
         // When multiple items are selected, clear the property editor
         _currentPropertyEditor.reset();
     }
+
+    updateEntryBoxSensitivity();
 }
 
 EntityInspector::PropertyParms EntityInspector::getPropertyParmsForKey(const std::string& key)
@@ -1591,8 +1606,8 @@ void EntityInspector::addClassProperties()
     }
 
     // Visit the entity class
-    eclass->forEachAttribute([&] (const EntityClassAttribute& a, bool) 
-    { 
+    eclass->forEachAttribute([&] (const EntityClassAttribute& a, bool)
+    {
         addClassAttribute(a);
     });
 }
@@ -1707,6 +1722,6 @@ void EntityInspector::toggle(const cmd::ArgumentList& args)
 }
 
 // Define the static EntityInspector module
-module::StaticModule<EntityInspector> entityInspectorModule;
+module::StaticModuleRegistration<EntityInspector> entityInspectorModule;
 
 } // namespace ui
