@@ -46,7 +46,7 @@ void GridUserInterface::initialiseModule(const IApplicationContext& ctx)
 	rMessage() << getName() << "::initialiseModule called." << std::endl;
 
 	// Add the grid status bar element
-	GlobalStatusBarManager().addTextElement("GridStatus", "grid_up.png", 
+	GlobalStatusBarManager().addTextElement("GridStatus", "grid_up.png",
 		statusbar::StandardPosition::GridSize, _("Current Grid Size"));
 	GlobalStatusBarManager().setText("GridStatus", getGridStatusText());
 
@@ -91,6 +91,6 @@ void GridUserInterface::toggleGrid(GridSize size, bool newState)
 	GlobalGrid().setGridSize(size);
 }
 
-module::StaticModule<GridUserInterface> gridUiModule;
+module::StaticModuleRegistration<GridUserInterface> gridUiModule;
 
 }
