@@ -29,6 +29,6 @@ void GroupDialogManager::initialiseModule(const IApplicationContext& ctx)
 	rMessage() << getName() << "::initialiseModule called" << std::endl;
 }
 
-module::StaticModule<GroupDialogManager> groupDialogManagerModule;
+module::StaticModuleRegistration<GroupDialogManager> groupDialogManagerModule;
 
 } // namespace ui
