@@ -1,10 +1,9 @@
 #pragma once
 
 #include "i18n.h"
-#include "ieventmanager.h"
-#include "imenumanager.h"
+#include "ui/imenumanager.h"
 #include "icommandsystem.h"
-#include "imainframe.h"
+#include "ui/imainframe.h"
 
 #include "wxutil/dialog/MessageBox.h"
 
