@@ -2,12 +2,10 @@
 
 #include "i18n.h"
 #include "selectionlib.h"
-#include "ibrush.h"
 #include "isound.h"
 #include "ui/iresourcechooser.h"
 #include "ui/idialogmanager.h"
 #include "entitylib.h" // EntityFindByClassnameWalker
-#include "selectionlib.h"
 #include "ientity.h" // Node_getEntity()
 #include "iregistry.h"
 #include "ui/imainframe.h"
@@ -19,6 +17,7 @@
 
 #include "ui/modelselector/ModelSelector.h"
 #include "ui/prefabselector/PrefabSelector.h"
+#include "ui/particles/ParticleChooserDialog.h"
 
 #include "string/convert.h"
 #include "scene/GroupNodeChecker.h"
@@ -43,25 +42,28 @@ namespace {
     const char* MODEL_CLASSNAME_STATIC = "func_static";
     const char* PLAYERSTART_CLASSNAME = "info_player_start";
 
-    const char* ADD_ENTITY_TEXT = N_("Create entity...");
+    const char* ADD_ENTITY_TEXT = N_("Create Entity...");
+    const char* CONVERT_TO_ENTITY_TEXT = N_("Convert to Entity...");
     const char* ADD_ENTITY_ICON = "cmenu_add_entity.png";
     const char* PLACE_PLAYERSTART_TEXT = N_("Place Player Start here");
     const char* PLACE_PLAYERSTART_ICON = "player_start16.png";
-    const char* ADD_MODEL_TEXT = N_("Create model...");
+    const char* ADD_MODEL_TEXT = N_("Create Model...");
     const char* ADD_MODEL_ICON = "cmenu_add_model.png";
-    const char* ADD_MONSTERCLIP_TEXT = N_("Surround with monsterclip");
+    const char* ADD_PARTICLE_TEXT = N_("Create Particle...");
+    const char* ADD_PARTICLE_ICON = "particle16.png";
+    const char* ADD_MONSTERCLIP_TEXT = N_("Surround with Monsterclip");
     const char* ADD_MONSTERCLIP_ICON = "monsterclip16.png";
-    const char* ADD_LIGHT_TEXT = N_("Create light...");
+    const char* ADD_LIGHT_TEXT = N_("Create Light...");
     const char* ADD_LIGHT_ICON = "cmenu_add_light.png";
-    const char* ADD_PREFAB_TEXT = N_("Insert prefab...");
+    const char* ADD_PREFAB_TEXT = N_("Insert Prefab...");
     const char* ADD_PREFAB_ICON = "cmenu_add_prefab.png";
-    const char* ADD_SPEAKER_TEXT = N_("Create speaker...");
+    const char* ADD_SPEAKER_TEXT = N_("Create Speaker...");
     const char* ADD_SPEAKER_ICON = "icon_sound.png";
     const char* CONVERT_TO_STATIC_TEXT = N_("Convert to func_static");
     const char* CONVERT_TO_STATIC_ICON = "cmenu_convert_static.png";
-    const char* REPARENT_PRIMITIVES_TEXT = N_("Reparent primitives");
-    const char* REVERT_TO_WORLDSPAWN_TEXT = N_("Revert to worldspawn");
-    const char* REVERT_TO_WORLDSPAWN_PARTIAL_TEXT = N_("Revert part to worldspawn");
+    const char* REPARENT_PRIMITIVES_TEXT = N_("Reparent Primitives");
+    const char* REVERT_TO_WORLDSPAWN_TEXT = N_("Revert to Worldspawn");
+    const char* REVERT_TO_WORLDSPAWN_PARTIAL_TEXT = N_("Revert Part to Worldspawn");
     const char* REVERT_TO_WORLDSPAWN_ICON = "cmenu_revert_worldspawn.png";
     const char* MERGE_ENTITIES_ICON = "cmenu_merge_entities.png";
     const char* MERGE_ENTITIES_TEXT = N_("Merge Entities");
@@ -70,7 +72,7 @@ namespace {
 }
 
 // Define the static OrthoContextMenu module
-module::StaticModule<OrthoContextMenu> orthoContextMenuModule;
+module::StaticModuleRegistration<OrthoContextMenu> orthoContextMenuModule;
 
 OrthoContextMenu& OrthoContextMenu::Instance()
 {
@@ -91,30 +93,32 @@ void OrthoContextMenu::Show(wxWindow* parent, const Vector3& point)
     analyseSelection();
 
     // Perform the visibility/sensitivity tests
-    for (MenuSections::const_iterator sec = _sections.begin(); sec != _sections.end(); ++sec)
+    for (const auto& [_, section] : _sections)
     {
-        for (MenuItems::const_iterator i = sec->second.begin(); i != sec->second.end(); ++i)
+        for (const auto& item : section)
         {
-            ui::IMenuItem& item = *(*i);
-
-            bool visible = item.isVisible();
+            bool visible = item->isVisible();
 
             if (visible)
             {
                 // Visibility check passed
                 // Run the preshow command
-                item.preShow();
+                item->preShow();
 
-                item.getMenuItem()->Enable(item.isSensitive());
+                item->getMenuItem()->Enable(item->isSensitive());
             }
             else
             {
                 // Visibility check failed, skip sensitivity check
-                item.getMenuItem()->Enable(false);
+                item->getMenuItem()->Enable(false);
             }
         }
     }
 
+    // Adjust the Create Entity label, it's Convert to Entity if anything is selected
+    _createEntityItem->getMenuItem()->SetItemLabel(
+        _selectionInfo.anythingSelected ? CONVERT_TO_ENTITY_TEXT : ADD_ENTITY_TEXT);
+
 	parent->PopupMenu(_widget.get());
 }
 
@@ -187,7 +191,7 @@ bool OrthoContextMenu::checkAddEntity()
     return !_selectionInfo.anythingSelected || _selectionInfo.onlyPrimitivesSelected;
 }
 
-bool OrthoContextMenu::checkAddModel()
+bool OrthoContextMenu::checkAddModelOrParticle()
 {
     return !_selectionInfo.anythingSelected;
 }
@@ -246,22 +250,25 @@ std::string OrthoContextMenu::getRegistryKeyWithDefault(
 void OrthoContextMenu::addEntity()
 {
     // Display the chooser to select an entity classname
-    std::string cName = wxutil::EntityClassChooser::chooseEntityClass();
+    auto purpose = _selectionInfo.anythingSelected ?
+        wxutil::EntityClassChooser::Purpose::ConvertEntity :
+        wxutil::EntityClassChooser::Purpose::AddEntity;
 
-    if (!cName.empty()) 
+    auto className = wxutil::EntityClassChooser::ChooseEntityClass(purpose);
+
+    if (className.empty()) return;
+
+    UndoableCommand command(_selectionInfo.anythingSelected ? "convertToEntity" : "createEntity");
+
+    // Create the entity. We might get an EntityCreationException if the
+    // wrong number of brushes is selected.
+    try
     {
-        UndoableCommand command("createEntity");
-
-        // Create the entity. We might get an EntityCreationException if the
-        // wrong number of brushes is selected.
-        try
-		{
-            GlobalEntityModule().createEntityFromSelection(cName, _lastPoint);
-        }
-        catch (cmd::ExecutionFailure& e)
-		{
-            wxutil::Messagebox::ShowError(e.what());
-        }
+        GlobalEntityModule().createEntityFromSelection(className, _lastPoint);
+    }
+    catch (cmd::ExecutionFailure& e)
+    {
+        wxutil::Messagebox::ShowError(e.what());
     }
 }
 
@@ -274,7 +281,7 @@ void OrthoContextMenu::callbackAddLight()
 {
     UndoableCommand command("addLight");
 
-    try 
+    try
 	{
         GlobalEntityModule().createEntityFromSelection(LIGHT_CLASSNAME, _lastPoint);
     }
@@ -291,7 +298,7 @@ void OrthoContextMenu::callbackAddPrefab()
     if (!result.prefabPath.empty())
     {
         // Pass the call to the map algorithm and give the lastPoint coordinate as argument
-        GlobalCommandSystem().executeCommand(LOAD_PREFAB_AT_CMD, 
+        GlobalCommandSystem().executeCommand(LOAD_PREFAB_AT_CMD,
             result.prefabPath, _lastPoint, result.insertAsGroup);
     }
 }
@@ -325,61 +332,69 @@ void OrthoContextMenu::callbackAddModel()
 {
     UndoableCommand command("addModel");
 
-    const SelectionInfo& info = GlobalSelectionSystem().getSelectionInfo();
+    // Display the model selector and block waiting for a selection (may be empty)
+    ModelSelectorResult ms = ModelSelector::chooseModel("", true, true);
 
-    // To create a model selection must be empty
-    if (info.totalCount == 0)
+    // If a model was selected, create the entity and set its model key
+    if (ms.model.empty())
     {
-        // Display the model selector and block waiting for a selection (may be empty)
-        ModelSelectorResult ms = ui::ModelSelector::chooseModel("", true, true);
+        return;
+    }
 
-        // If a model was selected, create the entity and set its model key
-        if (ms.model.empty())
+    try
+    {
+        auto modelDef = GlobalEntityClassManager().findModel(ms.model);
+
+        auto className = modelDef ? MODEL_CLASSNAME_ANIMATED : MODEL_CLASSNAME_STATIC;
+
+        auto modelNode = GlobalEntityModule().createEntityFromSelection(
+            className, _lastPoint
+        );
+
+        //Node_getTraversable(GlobalSceneGraph().root())->insert(modelNode);
+        modelNode->getEntity().setKeyValue("model", ms.model);
+        modelNode->getEntity().setKeyValue("skin", ms.skin);
+
+        // If 'createClip' is ticked, create a clip brush
+        if (ms.createClip)
         {
-            return;
-        }
-
-        try
-        {
-            auto modelDef = GlobalEntityClassManager().findModel(ms.model);
-
-            auto className = modelDef ? MODEL_CLASSNAME_ANIMATED : MODEL_CLASSNAME_STATIC;
-
-            auto modelNode = GlobalEntityModule().createEntityFromSelection(
-                className, _lastPoint
-            );
-
-            //Node_getTraversable(GlobalSceneGraph().root())->insert(modelNode);
-            modelNode->getEntity().setKeyValue("model", ms.model);
-            modelNode->getEntity().setKeyValue("skin", ms.skin);
-
-            // If 'createClip' is ticked, create a clip brush
-            if (ms.createClip)
-            {
-                GlobalCommandSystem().execute("SurroundWithMonsterclip");
-            }
-        }
-        catch (cmd::ExecutionFailure& e)
-        {
-            wxutil::Messagebox::ShowError(fmt::format(_("Unable to create model: {0}"), e.what()));
+            GlobalCommandSystem().execute("SurroundWithMonsterclip");
         }
     }
-    else
+    catch (cmd::ExecutionFailure& e)
     {
-        wxutil::Messagebox::ShowError(
-            _("Nothing must be selected for model creation")
-        );
+        wxutil::Messagebox::ShowError(fmt::format(_("Unable to create model: {0}"), e.what()));
+    }
+}
+
+void OrthoContextMenu::callbackAddParticle()
+{
+    UndoableCommand command("Create Particle");
+
+    // Display the particle selector and block waiting for a selection (may be empty)
+    auto result = ParticleChooserDialog::ChooseParticleAndEmitter();
+
+    if (result.selectedParticle.empty()) return;
+
+    try
+    {
+        auto node = GlobalEntityModule().createEntityFromSelection(result.selectedClassname, _lastPoint);
+
+        auto modelSpawnarg = result.selectedClassname == "func_smoke" ? "smoke" : "model";
+        node->getEntity().setKeyValue(modelSpawnarg, result.selectedParticle);
+    }
+    catch (cmd::ExecutionFailure& e)
+    {
+        wxutil::Messagebox::ShowError(fmt::format(_("Unable to create particle entity: {0}"), e.what()));
     }
 }
 
 void OrthoContextMenu::registerDefaultItems()
 {
-    wxutil::MenuItemPtr addEntity(
-        new wxutil::MenuItem(
-            new wxutil::IconTextMenuItem(_(ADD_ENTITY_TEXT), ADD_ENTITY_ICON),
-            std::bind(&OrthoContextMenu::addEntity, this),
-            std::bind(&OrthoContextMenu::checkAddEntity, this))
-    );
+    _createEntityItem = std::make_shared<wxutil::MenuItem>(
+        new wxutil::IconTextMenuItem(_(ADD_ENTITY_TEXT), ADD_ENTITY_ICON),
+        std::bind(&OrthoContextMenu::addEntity, this),
+        std::bind(&OrthoContextMenu::checkAddEntity, this));
 
     wxutil::MenuItemPtr addLight(
         new wxutil::MenuItem(
@@ -406,7 +421,14 @@ void OrthoContextMenu::registerDefaultItems()
         new wxutil::MenuItem(
 			new wxutil::IconTextMenuItem(_(ADD_MODEL_TEXT), ADD_MODEL_ICON),
             std::bind(&OrthoContextMenu::callbackAddModel, this),
-            std::bind(&OrthoContextMenu::checkAddModel, this))
+            std::bind(&OrthoContextMenu::checkAddModelOrParticle, this))
+    );
+
+    wxutil::MenuItemPtr addParticle(
+        new wxutil::MenuItem(
+            new wxutil::IconTextMenuItem(_(ADD_PARTICLE_TEXT), ADD_PARTICLE_ICON),
+            std::bind(&OrthoContextMenu::callbackAddParticle, this),
+            std::bind(&OrthoContextMenu::checkAddModelOrParticle, this))
     );
 
     wxutil::CommandMenuItemPtr surroundWithMonsterClip(
@@ -468,8 +490,9 @@ void OrthoContextMenu::registerDefaultItems()
     );
 
     // Register all constructed items
-    addItem(addEntity, SECTION_CREATE);
+    addItem(_createEntityItem, SECTION_CREATE);
     addItem(addModel, SECTION_CREATE);
+    addItem(addParticle, SECTION_CREATE);
     addItem(addLight, SECTION_CREATE);
     addItem(addSpeaker, SECTION_CREATE);
     addItem(addPrefab, SECTION_CREATE);
@@ -574,6 +597,7 @@ void OrthoContextMenu::initialiseModule(const IApplicationContext& ctx)
 
 void OrthoContextMenu::shutdownModule()
 {
+    _createEntityItem.reset();
 	_sections.clear();
     _widget.reset();
 }
