@@ -38,9 +38,6 @@ namespace {
 Brush::Brush(BrushNode& owner) :
     _owner(owner),
     _undoStateSaver(nullptr),
-    _faceCentroidPoints(GL_POINTS),
-    _uniqueVertexPoints(GL_POINTS),
-    _uniqueEdgePoints(GL_POINTS),
     m_planeChanged(false),
     m_transformChanged(false),
 	_detailFlag(Structural)
@@ -53,9 +50,6 @@ Brush::Brush(BrushNode& owner) :
 Brush::Brush(BrushNode& owner, const Brush& other) :
     _owner(owner),
     _undoStateSaver(nullptr),
-    _faceCentroidPoints(GL_POINTS),
-    _uniqueVertexPoints(GL_POINTS),
-    _uniqueEdgePoints(GL_POINTS),
     m_planeChanged(false),
     m_transformChanged(false),
 	_detailFlag(Structural)
@@ -202,6 +196,11 @@ void Brush::updateFaceVisibility()
     _owner.updateFaceVisibility();
 }
 
+void Brush::onFaceNeedsRenderableUpdate()
+{
+    _owner.onFaceNeedsRenderableUpdate();
+}
+
 Brush::DetailFlag Brush::getDetailFlag() const
 {
 	return _detailFlag;
@@ -263,25 +262,6 @@ const AABB& Brush::localAABB() const {
     return m_aabb_local;
 }
 
-void Brush::renderComponents(selection::ComponentSelectionMode mode, RenderableCollector& collector,
-	const VolumeTest& volume, const Matrix4& localToWorld) const
-{
-    switch (mode)
-	{
-        case selection::ComponentSelectionMode::Vertex:
-            collector.addRenderable(*m_state_point, _uniqueVertexPoints, localToWorld);
-            break;
-        case selection::ComponentSelectionMode::Edge:
-            collector.addRenderable(*m_state_point, _uniqueEdgePoints, localToWorld);
-            break;
-        case selection::ComponentSelectionMode::Face:
-            collector.addRenderable(*m_state_point, _faceCentroidPoints, localToWorld);
-            break;
-        default:
-            break;
-    }
-}
-
 void Brush::translate(const Vector3& translation)
 {
     std::for_each(m_faces.begin(), m_faces.end(),
@@ -384,15 +364,6 @@ FacePtr Brush::addPlane(const Vector3& p0, const Vector3& p1, const Vector3& p2,
 
 void Brush::setRenderSystem(const RenderSystemPtr& renderSystem)
 {
-    if (renderSystem)
-    {
-        m_state_point = renderSystem->capture("$POINT");
-    }
-    else
-    {
-        m_state_point.reset();
-    }
-
     for (Faces::iterator i = m_faces.begin(); i != m_faces.end(); ++i)
     {
         (*i)->setRenderSystem(renderSystem);
@@ -529,8 +500,6 @@ bool Brush::hasContributingFaces() const {
     return false;
 }
 
-/// \brief Removes faces that do not contribute to the brush. This is useful for cleaning up after CSG operations on the brush.
-/// Note: removal of empty faces is not performed during direct brush manipulations, because it would make a manipulation irreversible if it created an empty face.
 void Brush::removeEmptyFaces() {
     evaluateBRep();
 
@@ -546,6 +515,23 @@ void Brush::removeEmptyFaces() {
     }
 }
 
+void Brush::removeRedundantFaces()
+{
+    for (std::size_t i = 0; i < m_faces.size(); ++i)
+    {
+        auto& face = *m_faces[i];
+
+        if (face.plane3().isValid() && planeAlreadyDefined(i))
+        {
+            // Encountering a duplicated plane, this is possible after loading legacy maps
+            rWarning() << "Face plane " << face.plane3() << " already defined on this brush, discarding" << std::endl;
+
+            // Remove this face and continue with the same index (decrement to compensate loop increment)
+            erase(i--);
+        }
+    }
+}
+
 /// \brief Constructs \p winding from the intersection of \p plane with the other planes of the brush.
 void Brush::windingForClipPlane(Winding& winding, const Plane3& plane) const {
     FixedWinding buffer[2];
@@ -581,43 +567,6 @@ void Brush::windingForClipPlane(Winding& winding, const Plane3& plane) const {
     buffer[swap].writeToWinding(winding);
 }
 
-void Brush::update_wireframe(RenderableWireframe& wire, const bool* faces_visible) const
-{
-    wire.m_faceVertex.resize(_edgeIndices.size());
-    wire.m_vertices = _uniqueVertexPoints.size() > 0 ? &_uniqueVertexPoints.front() : NULL;
-    wire.m_size = 0;
-
-    for (std::size_t i = 0; i < _edgeFaces.size(); ++i)
-    {
-        if (faces_visible[_edgeFaces[i].first] || faces_visible[_edgeFaces[i].second])
-        {
-            wire.m_faceVertex[wire.m_size++] = _edgeIndices[i];
-        }
-    }
-}
-
-void Brush::update_faces_wireframe(RenderablePointVector& wire,
-                                   const std::size_t* visibleFaceIndices,
-                                   std::size_t numVisibleFaces) const
-{
-	if (numVisibleFaces > _faceCentroidPoints.size())
-	{
-		wire.clear();
-		return;
-	}
-
-    // Assure that the pointvector can carry as many faces as are visible
-    wire.resize(numVisibleFaces);
-
-    const std::size_t* visibleFaceIter = visibleFaceIndices;
-
-    // Pick all the visible face centroids from the vector
-    for (std::size_t i = 0; i < numVisibleFaces; ++i)
-    {
-        wire[i] = _faceCentroidPoints[*visibleFaceIter++];
-    }
-}
-
 /// \brief Makes this brush a deep-copy of the \p other.
 void Brush::copy(const Brush& other)
 {
@@ -976,6 +925,19 @@ bool Brush::plane_unique(std::size_t index) const {
     return true;
 }
 
+bool Brush::planeAlreadyDefined(std::size_t index) const
+{
+    for (std::size_t i = 0; index < m_faces.size() && i < index; ++i)
+    {
+        if (!plane3_inside(m_faces[index]->plane3(), m_faces[i]->plane3()))
+        {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 /// \brief Removes edges that are smaller than the tolerance used when generating brush windings.
 void Brush::removeDegenerateEdges() {
     for (std::size_t i = 0;  i < m_faces.size(); ++i) {
@@ -1104,40 +1066,42 @@ bool Brush::isBounded() {
     return true;
 }
 
-/// \brief Constructs the polygon windings for each face of the brush. Also updates the brush bounding-box and face texture-coordinates.
-bool Brush::buildWindings() {
+bool Brush::buildWindings()
+{
+    m_aabb_local = AABB();
+
+    for (std::size_t i = 0;  i < m_faces.size(); ++i)
     {
-        m_aabb_local = AABB();
+        auto& face = *m_faces[i];
 
-        for (std::size_t i = 0;  i < m_faces.size(); ++i) {
-            Face& f = *m_faces[i];
-
-            if (!f.plane3().isValid() || !plane_unique(i)) {
-                f.getWinding().resize(0);
-            }
-            else {
-                windingForClipPlane(f.getWinding(), f.plane3());
-
-                // update brush bounds
-                const Winding& winding = f.getWinding();
-
-                for (const WindingVertex& wv : winding)
-				{
-                    m_aabb_local.includePoint(wv.vertex);
-                }
-
-                // update texture coordinates
-                f.emitTextureCoordinates();
-            }
-
-            // greebo: Update the winding, now that it's constructed
-            f.updateWinding();
+        if (!face.plane3().isValid() || !plane_unique(i))
+        {
+            face.getWinding().resize(0);
         }
+        else
+        {
+            windingForClipPlane(face.getWinding(), face.plane3());
+
+            // update brush bounds
+            const auto& winding = face.getWinding();
+
+            for (const auto& vertex : winding)
+			{
+                m_aabb_local.includePoint(vertex.vertex);
+            }
+
+            // update texture coordinates
+            face.emitTextureCoordinates();
+        }
+
+        // greebo: Update the winding, now that it's constructed
+        face.updateWinding();
     }
 
     bool degenerate = !isBounded();
 
-    if (!degenerate) {
+    if (!degenerate)
+    {
         // clean up connectivity information.
         // these cleanups must be applied in a specific order.
         removeDegenerateEdges();
@@ -1298,7 +1262,7 @@ void Brush::buildBRep() {
             const Winding& w = m_faces[faceVertex.getFace()]->getWinding();
             Vector3 edge = math::midPoint(w[faceVertex.getVertex()].vertex,
                                           w[w.next(faceVertex.getVertex())].vertex);
-            _uniqueEdgePoints[i] = VertexCb(edge, colour_vertex);
+            _uniqueEdgePoints[i] = edge;
           }
         }
 
@@ -1348,7 +1312,7 @@ void Brush::buildBRep() {
             FaceVertexId faceVertex = faceVertices[ProximalVertexArray_index(vertexRings, uniqueVertices[i])];
 
             const Winding& winding = m_faces[faceVertex.getFace()]->getWinding();
-            _uniqueVertexPoints[i] = VertexCb(winding[faceVertex.getVertex()].vertex, colour_vertex);
+            _uniqueVertexPoints[i] = winding[faceVertex.getVertex()].vertex;
           }
         }
       }
@@ -1383,12 +1347,29 @@ void Brush::buildBRep() {
       for(std::size_t i=0; i<m_faces.size(); ++i)
       {
         m_faces[i]->construct_centroid();
-        _faceCentroidPoints[i] = VertexCb(m_faces[i]->centroid(), colour_vertex);
+        _faceCentroidPoints[i] = m_faces[i]->centroid();
       }
     }
   }
 }
 
+const std::vector<Vector3>& Brush::getVertices(selection::ComponentSelectionMode mode) const
+{
+    switch (mode)
+    {
+        case selection::ComponentSelectionMode::Vertex: 
+            return _uniqueVertexPoints;
+
+        case selection::ComponentSelectionMode::Edge: 
+            return _uniqueEdgePoints;
+
+        case selection::ComponentSelectionMode::Face: 
+            return _faceCentroidPoints;
+    }
+
+    throw std::runtime_error("Brush::getVertices: Component mode not supported");
+}
+
 // ----------------------------------------------------------------------------
 
 double Brush::m_maxWorldCoord = 0;
