@@ -148,14 +148,14 @@ void BrushModuleImpl::registerBrushCommands()
 	GlobalCommandSystem().addCommand("MakeVisportal", selection::algorithm::makeVisportal);
 	GlobalCommandSystem().addCommand("SurroundWithMonsterclip", selection::algorithm::surroundWithMonsterclip);
 
-	GlobalCommandSystem().addCommand("ResizeSelectedBrushesToBounds", selection::algorithm::resizeSelectedBrushesToBounds, 
+	GlobalCommandSystem().addCommand("ResizeSelectedBrushesToBounds", selection::algorithm::resizeSelectedBrushesToBounds,
 		{ cmd::ARGTYPE_VECTOR3, cmd::ARGTYPE_VECTOR3, cmd::ARGTYPE_STRING });
 }
 
 // -------------------------------------------------------------------------------------
 
 // Define a static BrushModule
-module::StaticModule<BrushModuleImpl> staticBrushModule;
+module::StaticModuleRegistration<BrushModuleImpl> staticBrushModule;
 
 }
 
