@@ -3,7 +3,6 @@
 #include "math/Plane3.h"
 #include "math/Line.h"
 #include "render.h"
-#include "render/VectorLightList.h"
 #include "ObservedSelectable.h"
 
 #include "SelectableComponents.h"
@@ -13,7 +12,7 @@
 
 typedef const Plane3* PlanePointer;
 typedef PlanePointer* PlanesIterator;
-class RenderableCollector;
+class IRenderableCollector;
 
 class FaceInstance;
 typedef std::list<FaceInstance*> FaceInstanceSet;
@@ -93,25 +92,9 @@ public:
 
 	void iterate_selected(AABB& aabb) const;
 
-	class RenderablePointVectorPushBack {
-		RenderablePointVector& m_points;
-	public:
-		RenderablePointVectorPushBack(RenderablePointVector& points) : m_points(points) {}
-		void operator()(const Vector3& point) const {
-			const Colour4b colour_selected(0, 0, 255, 255);
-			m_points.push_back(VertexCb(point, colour_selected));
-		}
-	};
-
-	void iterate_selected(RenderablePointVector& points) const;
-
 	bool intersectVolume(const VolumeTest& volume) const;
 	bool intersectVolume(const VolumeTest& volume, const Matrix4& localToWorld) const;
 
-	// Frontend render methods for submitting the face to the given collector
-	void renderWireframe(RenderableCollector& collector, const VolumeTest& volume,
-		const IRenderEntity& entity) const;
-
 	void testSelect(SelectionTest& test, SelectionIntersection& best);
 
 	void testSelect(Selector& selector, SelectionTest& test);
