@@ -6,37 +6,36 @@
 #include "string/convert.h"
 
 #include "string/predicate.h"
-#include <fmt/format.h>
 #include <functional>
 
 namespace eclass
 {
 
-const std::string EntityClass::DefaultWireShader("<0.3 0.3 1>");
-const std::string EntityClass::DefaultFillShader("(0.3 0.3 1)");
-const Vector3 EntityClass::DefaultEntityColour(0.3, 0.3, 1);
-const EntityClassAttribute EntityClass::_emptyAttribute("", "", "");
+namespace
+{
+    const Vector3 DefaultEntityColour(0.3, 0.3, 1);
+    const Vector4 UndefinedColour(-1, -1, -1, -1);
+}
 
-EntityClass::EntityClass(const std::string& name, const vfs::FileInfo& fileInfo) :
-    EntityClass(name, fileInfo, false)
+EntityClass::EntityClass(const std::string& name, bool fixedSize)
+: _name(name),
+  _visibility([this]() {
+      // Entity class visibility is NOT inherited -- hiding an abstract base entity from the list
+      // does not imply all of its concrete subclasses should also be hidden.
+      return getAttributeValue("editor_visibility", false) == "hidden" ? vfs::Visibility::HIDDEN
+                                                                       : vfs::Visibility::NORMAL;
+  }),
+  _colour(UndefinedColour),
+  _fixedSize(fixedSize)
 {}
 
 EntityClass::EntityClass(const std::string& name, const vfs::FileInfo& fileInfo, bool fixedSize)
-: _name(name),
-  _fileInfo(fileInfo),
-  _isLight(false),
-  _colour(-1, -1, -1),
-  _colourTransparent(false),
-  _fixedSize(fixedSize),
-  _model(""),
-  _skin(""),
-  _inheritanceResolved(false),
-  _modName("base"),
-  _parseStamp(0),
-  _blockChangeSignal(false)
-{}
+: EntityClass(name, fixedSize)
+{
+    _fileInfo = fileInfo;
+}
 
-std::string EntityClass::getName() const
+const std::string& EntityClass::getName() const
 {
     return _name;
 }
@@ -46,6 +45,11 @@ const IEntityClass* EntityClass::getParent() const
     return _parent;
 }
 
+vfs::Visibility EntityClass::getVisibility() const
+{
+    return _visibility.get();
+}
+
 sigc::signal<void>& EntityClass::changedSignal()
 {
     return _changedSignal;
@@ -59,8 +63,8 @@ bool EntityClass::isFixedSize() const
     else {
         // Check for the existence of editor_mins/maxs attributes, and that
         // they do not contain only a question mark
-        return (getAttribute("editor_mins").getValue().size() > 1
-                && getAttribute("editor_maxs").getValue().size() > 1);
+        return (getAttributeValue("editor_mins").size() > 1
+                && getAttributeValue("editor_maxs").size() > 1);
     }
 }
 
@@ -69,8 +73,8 @@ AABB EntityClass::getBounds() const
     if (isFixedSize())
     {
         return AABB::createFromMinMax(
-            string::convert<Vector3>(getAttribute("editor_mins").getValue()),
-            string::convert<Vector3>(getAttribute("editor_maxs").getValue())
+            string::convert<Vector3>(getAttributeValue("editor_mins")),
+            string::convert<Vector3>(getAttributeValue("editor_maxs"))
         );
     }
     else
@@ -91,24 +95,20 @@ void EntityClass::setIsLight(bool val)
         _fixedSize = true;
 }
 
-void EntityClass::setColour(const Vector3& colour)
+void EntityClass::setColour(const Vector4& colour)
 {
+    auto origColour = _colour;
     _colour = colour;
 
     // Set the entity colour to default, if none was specified
-    if (_colour == Vector3(-1, -1, -1))
+    if (_colour == UndefinedColour)
     {
         _colour = DefaultEntityColour;
     }
 
-    // Define fill and wire versions of the entity colour
-    _fillShader = _colourTransparent ?
-        fmt::format("[{0:f} {1:f} {2:f}]", _colour[0], _colour[1], _colour[2]) :
-        fmt::format("({0:f} {1:f} {2:f})", _colour[0], _colour[1], _colour[2]);
-
-    _wireShader = fmt::format("<{0:f} {1:f} {2:f}>", _colour[0], _colour[1], _colour[2]);
-
-    emitChangedSignal();
+    // Emit the signal if the colour actually changed
+    if (origColour != _colour)
+        emitChangedSignal();
 }
 
 void EntityClass::resetColour()
@@ -119,9 +119,14 @@ void EntityClass::resetColour()
         return;
 
     // Look for an editor_color on this class only
-    const EntityClassAttribute& attr = getAttribute("editor_color", false);
-    if (!attr.getValue().empty())
-        return setColour(string::convert<Vector3>(attr.getValue()));
+    const std::string colStr = getAttributeValue("editor_color", false);
+    if (!colStr.empty())
+    {
+        // Set alpha to 0.5 if editor_transparent is set
+        Vector4 colour(string::convert<Vector3>(colStr), _colourTransparent ? 0.5f : 1.0f);
+        setColour(colour);
+        return;
+    }
 
     // If there is a parent, inherit its getColour() directly, which takes into
     // account any EClassColourManager overrides at the parent level.
@@ -132,22 +137,11 @@ void EntityClass::resetColour()
     setColour(DefaultEntityColour);
 }
 
-const Vector3& EntityClass::getColour() const {
+const Vector4& EntityClass::getColour() const
+{
     return _colour;
 }
 
-const std::string& EntityClass::getWireShader() const
-{
-    // Use a fallback shader colour in case we don't have anything
-    return !_wireShader.empty() ? _wireShader : DefaultWireShader;
-}
-
-const std::string& EntityClass::getFillShader() const
-{
-    // Use a fallback shader colour in case we don't have anything
-    return !_fillShader.empty() ? _fillShader : DefaultFillShader;
-}
-
 /* ATTRIBUTES */
 
 /**
@@ -178,10 +172,14 @@ void EntityClass::emplaceAttribute(EntityClassAttribute&& attribute)
     }
 }
 
-EntityClass::Ptr EntityClass::create(const std::string& name, bool brushes)
+EntityClass::Ptr EntityClass::createDefault(const std::string& name, bool brushes)
 {
-    vfs::FileInfo emptyFileInfo("def/", "_autogenerated_by_darkradiant_.def", vfs::Visibility::HIDDEN);
-    return std::make_shared<EntityClass>(name, emptyFileInfo, !brushes);
+    auto eclass = std::make_shared<EntityClass>(name, !brushes);
+
+    // Force the entity class colour to default
+    eclass->setColour(UndefinedColour);
+
+    return eclass;
 }
 
 void EntityClass::forEachAttributeInternal(InternalAttrVisitor visitor,
@@ -238,12 +236,15 @@ void EntityClass::resolveInheritance(EntityClasses& classmap)
     // Lookup the parent name and return if it is not set. Also return if the
     // parent name is the same as our own classname, to avoid infinite
     // recursion.
-    std::string parName = getAttribute("inherit").getValue();
-    if (parName.empty() || parName == _name)
+    std::string parentName = getAttributeValue("inherit");
+    if (parentName.empty() || parentName == _name)
+    {
+        resetColour();
         return;
+    }
 
     // Find the parent entity class
-    EntityClasses::iterator pIter = classmap.find(parName);
+    auto pIter = classmap.find(parentName);
     if (pIter != classmap.end())
     {
         // Recursively resolve inheritance of parent
@@ -256,25 +257,25 @@ void EntityClass::resolveInheritance(EntityClasses& classmap)
     {
         rWarning() << "[eclassmgr] Entity class "
                               << _name << " specifies unknown parent class "
-                              << parName << std::endl;
+                              << parentName << std::endl;
     }
 
     // Set the resolved flag
     _inheritanceResolved = true;
 
-    if (!getAttribute("model").getValue().empty())
+    if (!getAttributeValue("model").empty())
     {
         // We have a model path (probably an inherited one)
-        setModelPath(getAttribute("model").getValue());
+        setModelPath(getAttributeValue("model"));
     }
 
-    if (getAttribute("editor_light").getValue() == "1" || getAttribute("spawnclass").getValue() == "idLight")
+    if (getAttributeValue("editor_light") == "1" || getAttributeValue("spawnclass") == "idLight")
     {
         // We have a light
         setIsLight(true);
     }
 
-    if (getAttribute("editor_transparent").getValue() == "1")
+    if (getAttributeValue("editor_transparent") == "1")
     {
         _colourTransparent = true;
     }
@@ -284,7 +285,12 @@ void EntityClass::resolveInheritance(EntityClasses& classmap)
     resetColour();
     if (_parent)
     {
-        _parent->changedSignal().connect(
+        // resolveInheritance() can be called more than once (e.g. after Reload Defs) so make sure
+        // we only have a single connection to the parent's changed signal.
+        if (_parentChangedConnection) {
+            _parentChangedConnection->disconnect();
+        }
+        _parentChangedConnection = _parent->changedSignal().connect(
             sigc::mem_fun(this, &EntityClass::resetColour)
         );
     }
@@ -307,37 +313,45 @@ bool EntityClass::isOfType(const std::string& className)
 
 std::string EntityClass::getDefFileName()
 {
-    return _fileInfo.fullPath();
+    return _fileInfo ? _fileInfo->fullPath() : "";
 }
 
 // Find a single attribute
-EntityClassAttribute& EntityClass::getAttribute(const std::string& name,
+EntityClassAttribute* EntityClass::getAttribute(const std::string& name,
                                                 bool includeInherited)
 {
-    return const_cast<EntityClassAttribute&>(
+    return const_cast<EntityClassAttribute*>(
         std::as_const(*this).getAttribute(name, includeInherited)
     );
 }
 
 // Find a single attribute
-const EntityClassAttribute& EntityClass::getAttribute(const std::string& name, bool includeInherited) const
+const EntityClassAttribute* EntityClass::getAttribute(const std::string& name, bool includeInherited) const
 {
     // First look up the attribute on this class; if found, we can simply return it
     auto f = _attributes.find(name);
     if (f != _attributes.end())
-        return f->second;
+        return &f->second;
 
     // If there is no parent or we have been instructed to ignore inheritance,
-    // this is the end of the line: return an empty attribute
+    // this is the end of the line: return nothing
     if (!_parent || !includeInherited)
-        return _emptyAttribute;
+        return nullptr;
 
     // Otherwise delegate to the parent (which will recurse until an attribute
     // is found or a null parent ends the process)
     return _parent->getAttribute(name);
 }
 
-const std::string& EntityClass::getAttributeType(const std::string& name) const
+std::string EntityClass::getAttributeValue(const std::string& name, bool includeInherited) const
+{
+    if (auto* attr = getAttribute(name, includeInherited); attr)
+        return attr->getValue();
+    else
+        return "";
+}
+
+std::string EntityClass::getAttributeType(const std::string& name) const
 {
     // Check the attributes on this class
     const auto& attribute = _attributes.find(name);
@@ -353,10 +367,10 @@ const std::string& EntityClass::getAttributeType(const std::string& name) const
     }
 
     // Walk up the inheritance tree until we spot a non-empty type
-    return _parent ? _parent->getAttributeType(name) : _emptyAttribute.getType();
+    return _parent ? _parent->getAttributeType(name) : "";
 }
 
-const std::string& EntityClass::getAttributeDescription(const std::string& name) const
+std::string EntityClass::getAttributeDescription(const std::string& name) const
 {
     // Check the attributes on this class first
     const auto& attribute = _attributes.find(name);
@@ -372,7 +386,7 @@ const std::string& EntityClass::getAttributeDescription(const std::string& name)
     }
 
     // Walk up the inheritance tree until we spot a non-empty description
-    return _parent ? _parent->getAttributeDescription(name) : _emptyAttribute.getDescription();
+    return _parent ? _parent->getAttributeDescription(name) : "";
 }
 
 void EntityClass::clear()
@@ -380,7 +394,7 @@ void EntityClass::clear()
     // Don't clear the name
     _isLight = false;
 
-    _colour = Vector3(-1,-1,-1);
+    _colour = UndefinedColour;
     _colourTransparent = false;
 
     _fixedSize = false;
@@ -465,21 +479,21 @@ void EntityClass::parseFromTokens(parser::DefTokeniser& tokeniser)
         }
 
         // We're only interested in non-inherited key/values when parsing
-        auto& attribute = getAttribute(key, false);
+        auto* attribute = getAttribute(key, false);
 
         // Add the EntityClassAttribute for this key/val
-        if (attribute.getType().empty())
+        if (!attribute)
         {
-            // Type is empty, attribute does not exist, add it.
-            // 
+            // Attribute does not exist, add it.
+            //
             // Following key-specific processing, add the keyvalue to the eclass
             // The type is an empty string, it will be set to a non-type as soon as we encounter it
             emplaceAttribute(EntityClassAttribute("", key, value, ""));
         }
-        else if (attribute.getValue().empty())
+        else if (attribute->getValue().empty())
         {
             // Attribute type is set, but value is empty, set the value.
-            attribute.setValue(value);
+            attribute->setValue(value);
         }
         else
         {
