@@ -19,12 +19,16 @@ EntityNode::EntityNode(const IEntityClassPtr& eclass) :
 	_eclass(eclass),
 	_spawnArgs(_eclass),
 	_namespaceManager(_spawnArgs),
+    _originKey(std::bind(&EntityNode::_originKeyChanged, this)),
 	_nameKey(_spawnArgs),
-	_renderableName(_nameKey),
+	_renderableName(*this, _nameKey),
+    _colourKey(std::bind(&EntityNode::_colourKeyChanged, this, std::placeholders::_1)),
 	_modelKey(*this),
 	_keyObservers(_spawnArgs),
 	_shaderParms(_keyObservers, _colourKey),
-	_direction(1,0,0)
+	_direction(1,0,0),
+    _isAttachedToRenderSystem(false),
+    _isShadowCasting(false)
 {
 }
 
@@ -39,12 +43,16 @@ EntityNode::EntityNode(const EntityNode& other) :
 	_spawnArgs(other._spawnArgs),
     _localToParent(other._localToParent),
 	_namespaceManager(_spawnArgs),
+    _originKey(std::bind(&EntityNode::_originKeyChanged, this)),
 	_nameKey(_spawnArgs),
-	_renderableName(_nameKey),
+	_renderableName(*this, _nameKey),
+    _colourKey(std::bind(&EntityNode::_colourKeyChanged, this, std::placeholders::_1)),
 	_modelKey(*this),
 	_keyObservers(_spawnArgs),
 	_shaderParms(_keyObservers, _colourKey),
-	_direction(1,0,0)
+	_direction(1,0,0),
+    _isAttachedToRenderSystem(false),
+    _isShadowCasting(false)
 {
 }
 
@@ -62,15 +70,21 @@ void EntityNode::construct()
 
 	TargetableNode::construct();
 
-	addKeyObserver("name", _nameKey);
-	addKeyObserver("_color", _colourKey);
+    // Observe basic keys
+    static_assert(std::is_base_of_v<sigc::trackable, NameKey>);
+    static_assert(std::is_base_of_v<sigc::trackable, ColourKey>);
+    static_assert(std::is_base_of_v<sigc::trackable, OriginKey>);
+    observeKey("origin", sigc::mem_fun(_originKey, &OriginKey::onKeyValueChanged));
+	observeKey("name", sigc::mem_fun(_nameKey, &NameKey::onKeyValueChanged));
+	observeKey("_color", sigc::mem_fun(_colourKey, &ColourKey::onKeyValueChanged));
 
-	_modelKeyObserver.setCallback(std::bind(&EntityNode::_modelKeyChanged, this, std::placeholders::_1));
-	addKeyObserver("model", _modelKeyObserver);
+    // Observe model-related keys
+    static_assert(std::is_base_of_v<sigc::trackable, EntityNode>);
+    static_assert(std::is_base_of_v<sigc::trackable, ModelKey>);
+	observeKey("model", sigc::mem_fun(this, &EntityNode::_modelKeyChanged));
+	observeKey("skin", sigc::mem_fun(_modelKey, &ModelKey::skinChanged));
 
-	// Connect the skin keyvalue change handler directly to the model node manager
-	_skinKeyObserver.setCallback(std::bind(&ModelKey::skinChanged, &_modelKey, std::placeholders::_1));
-	addKeyObserver("skin", _skinKeyObserver);
+    observeKey("noshadows", sigc::mem_fun(this, &EntityNode::_onNoShadowsSettingsChanged));
 
 	_shaderParms.addKeyObservers();
 
@@ -91,7 +105,7 @@ void EntityNode::constructClone(const EntityNode& original)
         if (originalModel && originalModel->hasModifiedScale())
         {
             assert(getModelKey().getNode()); // clone should have a child model like the original
-            auto transformable = Node_getTransformable(getModelKey().getNode());
+            auto transformable = scene::node_cast<ITransformable>(getModelKey().getNode());
 
             if (transformable)
             {
@@ -105,15 +119,13 @@ void EntityNode::constructClone(const EntityNode& original)
 
 void EntityNode::destruct()
 {
-	_shaderParms.removeKeyObservers();
+    // Disable model key handling, remove child nodes
+	_modelKey.destroy();
 
-	removeKeyObserver("skin", _skinKeyObserver);
-
-	_modelKey.setActive(false); // disable callbacks during destruction
-	removeKeyObserver("model", _modelKeyObserver);
-
-	removeKeyObserver("_color", _colourKey);
-	removeKeyObserver("name", _nameKey);
+    // Remove any attached entities before this EntityNode is defunct
+    // Particle node attachments might want to remove their renderables from
+    // this entity on destruction.
+    _attachedEnts.clear();
 
 	_eclassChangedConn.disconnect();
 
@@ -143,7 +155,7 @@ void EntityNode::createAttachedEntities()
             // Construct and store the attached entity
             auto attachedEnt = GlobalEntityModule().createEntity(cls);
             assert(attachedEnt);
-            _attachedEnts.push_back(attachedEnt);
+            _attachedEnts.push_back({attachedEnt, a.offset});
 
             // Set ourselves as the parent of the attached entity (for
             // localToParent transforms)
@@ -151,7 +163,7 @@ void EntityNode::createAttachedEntities()
 
             // Set the attached entity's transform matrix according to the
             // required offset
-            attachedEnt->localToParent() = Matrix4::getTranslation(a.offset);
+            attachedEnt->setLocalToParent(Matrix4::getTranslation(a.offset));
         }
     );
 }
@@ -159,11 +171,12 @@ void EntityNode::createAttachedEntities()
 void EntityNode::transformChanged()
 {
     Node::transformChanged();
+    TargetableNode::onTransformationChanged();
 
     // Broadcast transformChanged to all attached entities so they can update
     // their position
-    for (auto attached: _attachedEnts)
-        attached->transformChanged();
+    for (auto [node, offset]: _attachedEnts)
+        node->transformChanged();
 }
 
 void EntityNode::onEntityClassChanged()
@@ -175,14 +188,17 @@ void EntityNode::onEntityClassChanged()
     acquireShaders();
 }
 
-void EntityNode::addKeyObserver(const std::string& key, KeyObserver& observer)
+void EntityNode::observeKey(const std::string& key, KeyObserverFunc func)
 {
-	_keyObservers.insert(key, observer);
+    _keyObservers.observeKey(key, func);
 }
 
-void EntityNode::removeKeyObserver(const std::string& key, KeyObserver& observer)
+void EntityNode::foreachAttachment(const std::function<void(const IEntityNodePtr&)>& functor)
 {
-	_keyObservers.erase(key, observer);
+    for (const auto& [node, _] : _attachedEnts)
+    {
+        functor(node);
+    }
 }
 
 Entity& EntityNode::getEntity()
@@ -195,6 +211,11 @@ void EntityNode::refreshModel()
 	_modelKey.refreshModel();
 }
 
+std::string EntityNode::getEntityName() const
+{
+    return getName();
+}
+
 float EntityNode::getShaderParm(int parmNum) const
 {
 	return _shaderParms.getParmValue(parmNum);
@@ -205,11 +226,37 @@ const Vector3& EntityNode::getDirection() const
 	return _direction;
 }
 
+void EntityNode::addRenderable(const render::IRenderableObject::Ptr& object, Shader* shader)
+{
+    _renderObjects.addRenderable(object, shader);
+}
+
+void EntityNode::removeRenderable(const render::IRenderableObject::Ptr& object)
+{
+    _renderObjects.removeRenderable(object);
+}
+
+void EntityNode::foreachRenderable(const IRenderEntity::ObjectVisitFunction& functor)
+{
+    _renderObjects.foreachRenderable(functor);
+}
+
+void EntityNode::foreachRenderableTouchingBounds(const AABB& bounds,
+    const ObjectVisitFunction& functor)
+{
+    _renderObjects.foreachRenderableTouchingBounds(bounds, functor);
+}
+
+bool EntityNode::isShadowCasting() const
+{
+    return _isShadowCasting;
+}
+
 std::string EntityNode::getFingerprint()
 {
     std::map<std::string, std::string> sortedKeyValues;
 
-    // Entities are just a collection of key/value pairs, 
+    // Entities are just a collection of key/value pairs,
     // use them in lower case form, ignore inherited keys, sort before hashing
     _spawnArgs.forEachKeyValue([&](const std::string& key, const std::string& value)
     {
@@ -299,6 +346,13 @@ void EntityNode::onInsertIntoScene(scene::IMapRootNode& root)
 	_spawnArgs.connectUndoSystem(root.getUndoSystem());
 	_modelKey.connectUndoSystem(root.getUndoSystem());
 
+    attachToRenderSystem();
+
+    for (const auto& [node, _] : _attachedEnts)
+    {
+        node->onInsertIntoScene(root);
+    }
+
 	SelectableNode::onInsertIntoScene(root);
     TargetableNode::onInsertIntoScene(root);
 }
@@ -308,6 +362,13 @@ void EntityNode::onRemoveFromScene(scene::IMapRootNode& root)
     TargetableNode::onRemoveFromScene(root);
 	SelectableNode::onRemoveFromScene(root);
 
+    detachFromRenderSystem();
+
+    for (const auto& [node, _] : _attachedEnts)
+    {
+        node->onRemoveFromScene(root);
+    }
+
 	_modelKey.disconnectUndoSystem(root.getUndoSystem());
 	_spawnArgs.disconnectUndoSystem(root.getUndoSystem());
 
@@ -344,7 +405,7 @@ void EntityNode::onChildRemoved(const scene::INodePtr& child)
 
 std::string EntityNode::name() const
 {
-	return _nameKey.name();
+	return _nameKey.getName();
 }
 
 scene::INode::Type EntityNode::getNodeType() const
@@ -352,28 +413,27 @@ scene::INode::Type EntityNode::getNodeType() const
 	return Type::Entity;
 }
 
-void EntityNode::renderSolid(RenderableCollector& collector,
-                             const VolumeTest& volume) const
+void EntityNode::onPreRender(const VolumeTest& volume)
 {
-    // Render any attached entities
-    renderAttachments(
-        [&](const scene::INodePtr& n) { n->renderSolid(collector, volume); }
-    );
-}
-
-void EntityNode::renderWireframe(RenderableCollector& collector,
-                                 const VolumeTest& volume) const
-{
-	// Submit renderable text name if required
-	if (EntitySettings::InstancePtr()->getRenderEntityNames())
+    if (EntitySettings::InstancePtr()->getRenderEntityNames())
     {
-        collector.addRenderable(*getWireShader(), _renderableName,
-                                localToWorld());
+        _renderableName.update(_textRenderer);
+    }
+    else
+    {
+        _renderableName.clear();
     }
 
     // Render any attached entities
     renderAttachments(
-        [&](const scene::INodePtr& n) { n->renderWireframe(collector, volume); }
+        [&](const scene::INodePtr& n) { n->onPreRender(volume); }
+    );
+}
+
+void EntityNode::renderHighlights(IRenderableCollector& collector, const VolumeTest& volume)
+{
+    renderAttachments(
+        [&](const scene::INodePtr& n) { n->renderHighlights(collector, volume); }
     );
 }
 
@@ -386,28 +446,43 @@ void EntityNode::acquireShaders(const RenderSystemPtr& renderSystem)
 {
     if (renderSystem)
     {
-        _fillShader = renderSystem->capture(_spawnArgs.getEntityClass()->getFillShader());
-        _wireShader = renderSystem->capture(_spawnArgs.getEntityClass()->getWireShader());
+        const auto& colour = _spawnArgs.getEntityClass()->getColour();
+        _fillShader = renderSystem->capture(ColourShaderType::CameraSolid, colour);
+        _wireShader = renderSystem->capture(ColourShaderType::OrthoviewSolid, colour);
+        _colourShader = renderSystem->capture(ColourShaderType::CameraAndOrthoview, colour);
+        _textRenderer = renderSystem->captureTextRenderer(IGLFont::Style::Sans, 14);
     }
     else
     {
         _fillShader.reset();
         _wireShader.reset();
+        _colourShader.reset();
+        _textRenderer.reset();
     }
 }
 
 void EntityNode::setRenderSystem(const RenderSystemPtr& renderSystem)
 {
+    // Detach from any existing render system first
+    detachFromRenderSystem();
+
 	SelectableNode::setRenderSystem(renderSystem);
 
+    if (renderSystem)
+    {
+        attachToRenderSystem();
+    }
+
     acquireShaders(renderSystem);
 
 	// The colour key is maintaining a shader object as well
 	_colourKey.setRenderSystem(renderSystem);
 
     // Make sure any attached entities have a render system too
-    for (IEntityNodePtr node: _attachedEnts)
+    for (auto [node, offset]: _attachedEnts)
         node->setRenderSystem(renderSystem);
+
+	TargetableNode::onRenderSystemChanged();
 }
 
 std::size_t EntityNode::getHighlightFlags()
@@ -417,6 +492,24 @@ std::size_t EntityNode::getHighlightFlags()
 	return isGroupMember() ? (Highlight::Selected | Highlight::GroupMember) : Highlight::Selected;
 }
 
+void EntityNode::onVisibilityChanged(bool isVisibleNow)
+{
+    SelectableNode::onVisibilityChanged(isVisibleNow);
+    TargetableNode::onVisibilityChanged(isVisibleNow);
+
+    for (const auto& [node, _] : _attachedEnts)
+    {
+        if (isVisibleNow)
+        {
+            scene::showSubgraph(node);
+        }
+        else
+        {
+            scene::hideSubgraph(node);
+        }
+    }
+}
+
 ModelKey& EntityNode::getModelKey()
 {
 	return _modelKey;
@@ -440,16 +533,41 @@ void EntityNode::_modelKeyChanged(const std::string& value)
 	onModelKeyChanged(value);
 }
 
+void EntityNode::_originKeyChanged()
+{
+    // TODO: add virtual callout for subclasses
+}
+
+void EntityNode::_colourKeyChanged(const std::string& value)
+{
+    onColourKeyChanged(value);
+}
+
+void EntityNode::_onNoShadowsSettingsChanged(const std::string& noShadowsValue)
+{
+    _isShadowCasting = noShadowsValue != "1";
+}
+
 const ShaderPtr& EntityNode::getWireShader() const
 {
 	return _wireShader;
 }
 
+const ShaderPtr& EntityNode::getColourShader() const
+{
+	return _colourShader;
+}
+
 const ShaderPtr& EntityNode::getFillShader() const
 {
 	return _fillShader;
 }
 
+Vector4 EntityNode::getEntityColour() const
+{
+    return _spawnArgs.getEntityClass()->getColour();
+}
+
 void EntityNode::onPostUndo()
 {
 	// After undo operations there might remain some child nodes
@@ -472,4 +590,38 @@ void EntityNode::onPostRedo()
 	});
 }
 
+void EntityNode::onEntitySettingsChanged()
+{
+    if (!EntitySettings::InstancePtr()->getRenderEntityNames())
+    {
+        _renderableName.clear();
+    }
+}
+
+void EntityNode::attachToRenderSystem()
+{
+    if (_isAttachedToRenderSystem) return;
+
+    auto renderSystem = getRenderSystem();
+
+    if (renderSystem)
+    {
+        renderSystem->addEntity(std::dynamic_pointer_cast<IRenderEntity>(shared_from_this()));
+        _isAttachedToRenderSystem = true;
+    }
+}
+
+void EntityNode::detachFromRenderSystem()
+{
+    if (!_isAttachedToRenderSystem) return;
+
+    auto renderSystem = getRenderSystem();
+
+    if (renderSystem)
+    {
+        renderSystem->removeEntity(std::dynamic_pointer_cast<IRenderEntity>(shared_from_this()));
+        _isAttachedToRenderSystem = false;
+    }
+}
+
 } // namespace entity
