@@ -13,11 +13,9 @@ GenericEntityNode::GenericEntityNode(const IEntityClassPtr& eclass) :
 	m_angleKey(std::bind(&GenericEntityNode::angleChanged, this)),
 	m_angle(AngleKey::IDENTITY),
 	m_rotationKey(std::bind(&GenericEntityNode::rotationChanged, this)),
-	m_arrow(m_ray),
-	m_aabb_solid(m_aabb_local),
-	m_aabb_wire(m_aabb_local),
-	_allow3Drotations(_spawnArgs.getKeyValue("editor_rotatable") == "1"),
-    _solidAABBRenderMode(SolidBoxes)
+    _renderableArrow(*this),
+    _renderableBox(*this, localAABB(), worldAABB().getOrigin()),
+	_allow3Drotations(_spawnArgs.getKeyValue("editor_rotatable") == "1")
 {}
 
 GenericEntityNode::GenericEntityNode(const GenericEntityNode& other) :
@@ -28,33 +26,14 @@ GenericEntityNode::GenericEntityNode(const GenericEntityNode& other) :
 	m_angleKey(std::bind(&GenericEntityNode::angleChanged, this)),
 	m_angle(AngleKey::IDENTITY),
 	m_rotationKey(std::bind(&GenericEntityNode::rotationChanged, this)),
-	m_arrow(m_ray),
-	m_aabb_solid(m_aabb_local),
-	m_aabb_wire(m_aabb_local),
-	_allow3Drotations(_spawnArgs.getKeyValue("editor_rotatable") == "1"),
-    _solidAABBRenderMode(other._solidAABBRenderMode)
+    _renderableArrow(*this),
+    _renderableBox(*this, localAABB(), worldAABB().getOrigin()),
+	_allow3Drotations(_spawnArgs.getKeyValue("editor_rotatable") == "1")
 {}
 
-GenericEntityNode::~GenericEntityNode()
+std::shared_ptr<GenericEntityNode> GenericEntityNode::Create(const IEntityClassPtr& eclass)
 {
-	if (!_allow3Drotations)
-	{
-		// Ordinary rotation (2D around z axis), use angle key observer
-		removeKeyObserver("angle", _angleObserver);
-	}
-	else
-	{
-		// Full 3D rotations allowed, observe both keys using the rotation key observer
-		removeKeyObserver("angle", _angleObserver);
-		removeKeyObserver("rotation", _rotationObserver);
-	}
-
-	removeKeyObserver("origin", m_originKey);
-}
-
-GenericEntityNodePtr GenericEntityNode::Create(const IEntityClassPtr& eclass)
-{
-	GenericEntityNodePtr instance(new GenericEntityNode(eclass));
+	auto instance = std::make_shared<GenericEntityNode>(eclass);
 	instance->construct();
 
 	return instance;
@@ -71,22 +50,20 @@ void GenericEntityNode::construct()
 
 	if (!_allow3Drotations)
 	{
-		_angleObserver.setCallback(std::bind(&AngleKey::angleChanged, &m_angleKey, std::placeholders::_1));
-
 		// Ordinary rotation (2D around z axis), use angle key observer
-		addKeyObserver("angle", _angleObserver);
+        static_assert(std::is_base_of_v<sigc::trackable, AngleKey>);
+		observeKey("angle", sigc::mem_fun(m_angleKey, &AngleKey::angleChanged));
 	}
 	else
 	{
-		_angleObserver.setCallback(std::bind(&RotationKey::angleChanged, &m_rotationKey, std::placeholders::_1));
-		_rotationObserver.setCallback(std::bind(&RotationKey::rotationChanged, &m_rotationKey, std::placeholders::_1));
-
 		// Full 3D rotations allowed, observe both keys using the rotation key observer
-		addKeyObserver("angle", _angleObserver);
-		addKeyObserver("rotation", _rotationObserver);
-	}
+        static_assert(std::is_base_of_v<sigc::trackable, RotationKey>);
+		observeKey("angle", sigc::mem_fun(m_rotationKey, &RotationKey::angleChanged));
+        observeKey("rotation", sigc::mem_fun(m_rotationKey, &RotationKey::rotationChanged));
+    }
 
-	addKeyObserver("origin", m_originKey);
+    static_assert(std::is_base_of_v<sigc::trackable, OriginKey>);
+	observeKey("origin", sigc::mem_fun(m_originKey, &OriginKey::onKeyValueChanged));
 }
 
 void GenericEntityNode::snapto(float snap)
@@ -115,44 +92,36 @@ void GenericEntityNode::testSelect(Selector& selector, SelectionTest& test)
 
 scene::INodePtr GenericEntityNode::clone() const
 {
-	GenericEntityNodePtr node(new GenericEntityNode(*this));
+	auto node = std::shared_ptr<GenericEntityNode>(new GenericEntityNode(*this));
 	node->construct();
     node->constructClone(*this);
 
 	return node;
 }
 
-GenericEntityNode::SolidAAABBRenderMode GenericEntityNode::getSolidAABBRenderMode() const
+void GenericEntityNode::onPreRender(const VolumeTest& volume)
 {
-    return _solidAABBRenderMode;
+    EntityNode::onPreRender(volume);
+
+    _renderableBox.update(getColourShader());
+    _renderableArrow.update(getColourShader());
 }
 
-void GenericEntityNode::renderArrow(const ShaderPtr& shader, RenderableCollector& collector,
-	const VolumeTest& volume, const Matrix4& localToWorld) const
+void GenericEntityNode::renderHighlights(IRenderableCollector& collector, const VolumeTest& volume)
 {
-	if (EntitySettings::InstancePtr()->getShowEntityAngles())
-	{
-		collector.addRenderable(*shader, m_arrow, localToWorld);
-	}
+    EntityNode::renderHighlights(collector, volume);
+
+    collector.addHighlightRenderable(_renderableArrow, Matrix4::getIdentity());
+    collector.addHighlightRenderable(_renderableBox, Matrix4::getIdentity());
 }
 
-void GenericEntityNode::renderSolid(RenderableCollector& collector, const VolumeTest& volume) const
+void GenericEntityNode::setRenderSystem(const RenderSystemPtr& renderSystem)
 {
-	EntityNode::renderSolid(collector, volume);
+    EntityNode::setRenderSystem(renderSystem);
 
-	const ShaderPtr& shader = getSolidAABBRenderMode() == GenericEntityNode::WireFrameOnly ?
-		getWireShader() : getFillShader();
-
-	collector.addRenderable(*shader, m_aabb_solid, localToWorld());
-	renderArrow(shader, collector, volume, localToWorld());
-}
-
-void GenericEntityNode::renderWireframe(RenderableCollector& collector, const VolumeTest& volume) const
-{
-	EntityNode::renderWireframe(collector, volume);
-
-	collector.addRenderable(*getWireShader(), m_aabb_wire, localToWorld());
-	renderArrow(getWireShader(), collector, volume, localToWorld());
+    // Clear the geometry from any previous shader
+    _renderableBox.clear();
+    _renderableArrow.clear();
 }
 
 const Vector3& GenericEntityNode::getDirection() const
@@ -205,7 +174,7 @@ void GenericEntityNode::_freezeTransform()
 
 void GenericEntityNode::updateTransform()
 {
-	localToParent() = Matrix4::getTranslation(m_origin);
+	setLocalToParent(Matrix4::getTranslation(m_origin));
 
 	if (_allow3Drotations)
 	{
@@ -218,6 +187,8 @@ void GenericEntityNode::updateTransform()
                                   .transformDirection(Vector3(1, 0, 0));
     }
 
+    _renderableBox.queueUpdate();
+    _renderableArrow.queueUpdate();
 	transformChanged();
 }
 
@@ -252,11 +223,16 @@ const Vector3& GenericEntityNode::getUntransformedOrigin()
     return m_originKey.get();
 }
 
+const Vector3& GenericEntityNode::getWorldPosition() const
+{
+    return m_origin;
+}
+
 void GenericEntityNode::onChildAdded(const scene::INodePtr& child)
 {
     EntityNode::onChildAdded(child);
 
-    _solidAABBRenderMode = SolidBoxes;
+    _renderableBox.setFillMode(true);
 
     // Check if this node has any actual models/particles as children
     Node::foreachNode([&](const scene::INodePtr& node)
@@ -264,7 +240,7 @@ void GenericEntityNode::onChildAdded(const scene::INodePtr& child)
         // We consider all non-path-connection childnodes as "models"
         if (child->getNodeType() != scene::INode::Type::EntityConnection)
         {
-            _solidAABBRenderMode = WireFrameOnly;
+            _renderableBox.setFillMode(false);
             return false; // stop traversal
         }
 
@@ -276,7 +252,7 @@ void GenericEntityNode::onChildRemoved(const scene::INodePtr& child)
 {
     EntityNode::onChildRemoved(child);
 
-    _solidAABBRenderMode = SolidBoxes;
+    _renderableBox.setFillMode(true);
 
     // Check if this node has any actual models/particles as children
     Node::foreachNode([&](const scene::INodePtr& node)
@@ -285,7 +261,7 @@ void GenericEntityNode::onChildRemoved(const scene::INodePtr& child)
         // Ignore the child itself as this event is raised before the node is actually removed.
         if (node != child && child->getNodeType() != scene::INode::Type::EntityConnection)
         {
-            _solidAABBRenderMode = WireFrameOnly;
+            _renderableBox.setFillMode(false);
             return false; // stop traversal
         }
 
@@ -293,6 +269,40 @@ void GenericEntityNode::onChildRemoved(const scene::INodePtr& child)
     });
 }
 
+void GenericEntityNode::onInsertIntoScene(scene::IMapRootNode& root)
+{
+    // Call the base class first
+    EntityNode::onInsertIntoScene(root);
+
+    _renderableBox.queueUpdate();
+    _renderableArrow.queueUpdate();
+}
+
+void GenericEntityNode::onRemoveFromScene(scene::IMapRootNode& root)
+{
+    // Call the base class first
+    EntityNode::onRemoveFromScene(root);
+
+    _renderableBox.clear();
+    _renderableArrow.clear();
+}
+
+void GenericEntityNode::onVisibilityChanged(bool isVisibleNow)
+{
+    EntityNode::onVisibilityChanged(isVisibleNow);
+
+    if (isVisibleNow)
+    {
+        _renderableBox.queueUpdate();
+        _renderableArrow.queueUpdate();
+    }
+    else
+    {
+        _renderableBox.clear();
+        _renderableArrow.clear();
+    }
+}
+
 void GenericEntityNode::originChanged()
 {
 	m_origin = m_originKey.get();
