@@ -17,16 +17,11 @@ namespace
 SpeakerNode::SpeakerNode(const IEntityClassPtr& eclass) :
 	EntityNode(eclass),
 	m_originKey(std::bind(&SpeakerNode::originChanged, this)),
-	m_origin(ORIGINKEY_IDENTITY),
-	_renderableRadii(m_origin, _radiiTransformed),
-	m_useSpeakerRadii(true),
-	m_minIsSet(false),
-	m_maxIsSet(false),
-	m_aabb_solid(m_aabb_local),
-	m_aabb_wire(m_aabb_local),
-	_radiusMinObserver(std::bind(&SpeakerNode::sMinChanged, this, std::placeholders::_1)),
-	_radiusMaxObserver(std::bind(&SpeakerNode::sMaxChanged, this, std::placeholders::_1)),
-	_shaderObserver(std::bind(&SpeakerNode::sShaderChanged, this, std::placeholders::_1)),
+    _renderableBox(*this, m_aabb_local, worldAABB().getOrigin()),
+    _renderableRadiiWireframe(*this, m_origin, _radiiTransformed),
+    _renderableRadiiFill(*this, m_origin, _radiiTransformed),
+    _renderableRadiiFillOutline(*this, m_origin, _radiiTransformed),
+    _showRadiiWhenUnselected(EntitySettings::InstancePtr()->getShowAllSpeakerRadii()),
 	_dragPlanes(std::bind(&SpeakerNode::selectedChangedComponent, this, std::placeholders::_1))
 {}
 
@@ -34,16 +29,11 @@ SpeakerNode::SpeakerNode(const SpeakerNode& other) :
 	EntityNode(other),
 	Snappable(other),
 	m_originKey(std::bind(&SpeakerNode::originChanged, this)),
-	m_origin(ORIGINKEY_IDENTITY),
-	_renderableRadii(m_origin, _radiiTransformed),
-	m_useSpeakerRadii(true),
-	m_minIsSet(false),
-	m_maxIsSet(false),
-	m_aabb_solid(m_aabb_local),
-	m_aabb_wire(m_aabb_local),
-	_radiusMinObserver(std::bind(&SpeakerNode::sMinChanged, this, std::placeholders::_1)),
-	_radiusMaxObserver(std::bind(&SpeakerNode::sMaxChanged, this, std::placeholders::_1)),
-	_shaderObserver(std::bind(&SpeakerNode::sShaderChanged, this, std::placeholders::_1)),
+    _renderableBox(*this, m_aabb_local, worldAABB().getOrigin()),
+    _renderableRadiiWireframe(*this, m_origin, _radiiTransformed),
+    _renderableRadiiFill(*this, m_origin, _radiiTransformed),
+    _renderableRadiiFillOutline(*this, m_origin, _radiiTransformed),
+    _showRadiiWhenUnselected(EntitySettings::InstancePtr()->getShowAllSpeakerRadii()),
 	_dragPlanes(std::bind(&SpeakerNode::selectedChangedComponent, this, std::placeholders::_1))
 {}
 
@@ -62,18 +52,17 @@ void SpeakerNode::construct()
 	m_aabb_local = _spawnArgs.getEntityClass()->getBounds();
 	m_aabb_border = m_aabb_local;
 
-	addKeyObserver("origin", m_originKey);
-	addKeyObserver(KEY_S_SHADER, _shaderObserver);
-	addKeyObserver(KEY_S_MINDISTANCE, _radiusMinObserver);
-	addKeyObserver(KEY_S_MAXDISTANCE, _radiusMaxObserver);
+	observeKey("origin", sigc::mem_fun(m_originKey, &OriginKey::onKeyValueChanged));
+
+    // Observe speaker-related spawnargs
+    static_assert(std::is_base_of<sigc::trackable, SpeakerNode>::value);
+	observeKey(KEY_S_SHADER, sigc::mem_fun(this, &SpeakerNode::sShaderChanged));
+	observeKey(KEY_S_MINDISTANCE, sigc::mem_fun(this, &SpeakerNode::sMinChanged));
+	observeKey(KEY_S_MAXDISTANCE, sigc::mem_fun(this, &SpeakerNode::sMaxChanged));
 }
 
 SpeakerNode::~SpeakerNode()
 {
-	removeKeyObserver("origin", m_originKey);
-	removeKeyObserver(KEY_S_SHADER, _shaderObserver);
-	removeKeyObserver(KEY_S_MINDISTANCE, _radiusMinObserver);
-	removeKeyObserver(KEY_S_MAXDISTANCE, _radiusMaxObserver);
 }
 
 void SpeakerNode::originChanged()
@@ -139,6 +128,7 @@ void SpeakerNode::sMinChanged(const std::string& value)
 	_radiiTransformed.setMin(_radii.getMin());
 
 	updateAABB();
+    updateRenderables();
 }
 
 void SpeakerNode::sMaxChanged(const std::string& value)
@@ -159,16 +149,15 @@ void SpeakerNode::sMaxChanged(const std::string& value)
 	_radiiTransformed.setMax(_radii.getMax());
 
 	updateAABB();
+    updateRenderables();
 }
 
-// Snappable implementation
 void SpeakerNode::snapto(float snap)
 {
 	m_originKey.snap(snap);
 	m_originKey.write(_spawnArgs);
 }
 
-// Bounded implementation
 const AABB& SpeakerNode::localAABB() const
 {
 	return m_aabb_border;
@@ -242,34 +231,59 @@ scene::INodePtr SpeakerNode::clone() const
 	return node;
 }
 
-/* Renderable implementation */
-
-void SpeakerNode::renderSolid(RenderableCollector& collector,
-                              const VolumeTest& volume) const
+void SpeakerNode::onPreRender(const VolumeTest& volume)
 {
-	EntityNode::renderSolid(collector, volume);
+    EntityNode::onPreRender(volume);
 
-	collector.addRenderable(*getFillShader(), m_aabb_solid, localToWorld());
+    _renderableBox.update(getColourShader());
 
-    // Submit the speaker radius if we are selected or the "show all speaker
-    // radii" option is set
-	if (isSelected() || EntitySettings::InstancePtr()->getShowAllSpeakerRadii())
+    if (_showRadiiWhenUnselected || isSelected())
     {
-		collector.addRenderable(*getFillShader(), _renderableRadii, localToWorld());
+        _renderableRadiiWireframe.update(getWireShader());
+        _renderableRadiiFill.update(_radiiFillShader);
+        _renderableRadiiFillOutline.update(_radiiFillOutlineShader);
+    }
+    else
+    {
+        _renderableRadiiWireframe.clear();
+        _renderableRadiiFill.clear();
+        _renderableRadiiFillOutline.clear();
     }
 }
-void SpeakerNode::renderWireframe(RenderableCollector& collector,
-                                  const VolumeTest& volume) const
+
+void SpeakerNode::renderHighlights(IRenderableCollector& collector, const VolumeTest& volume)
 {
-	EntityNode::renderWireframe(collector, volume);
+    collector.addHighlightRenderable(_renderableBox, Matrix4::getIdentity());
 
-	collector.addRenderable(*getWireShader(), m_aabb_wire, localToWorld());
-
-    // Submit the speaker radius if we are selected or the "show all speaker
-    // radii" option is set
-	if (isSelected() || EntitySettings::InstancePtr()->getShowAllSpeakerRadii())
+    if (!collector.supportsFullMaterials())
     {
-		collector.addRenderable(*getWireShader(), _renderableRadii, localToWorld());
+        collector.addHighlightRenderable(_renderableRadiiWireframe, Matrix4::getIdentity());
+    }
+    else
+    {
+        collector.addHighlightRenderable(_renderableRadiiFill, Matrix4::getIdentity());
+    }
+
+    EntityNode::renderHighlights(collector, volume);
+}
+
+void SpeakerNode::setRenderSystem(const RenderSystemPtr& renderSystem)
+{
+    EntityNode::setRenderSystem(renderSystem);
+
+    // Clear the geometry from any previous shader
+    clearRenderables();
+
+    if (renderSystem)
+    {
+        auto renderColour = getEntityColour();
+        _radiiFillShader = renderSystem->capture(ColourShaderType::CameraTranslucent, renderColour);
+        _radiiFillOutlineShader = renderSystem->capture(ColourShaderType::CameraOutline, renderColour);
+    }
+    else
+    {
+        _radiiFillShader.reset();
+        _radiiFillOutlineShader.reset();
     }
 }
 
@@ -280,8 +294,10 @@ void SpeakerNode::translate(const Vector3& translation)
 
 void SpeakerNode::updateTransform()
 {
-	localToParent() = Matrix4::getTranslation(m_origin);
+	setLocalToParent(Matrix4::getTranslation(m_origin));
 	transformChanged();
+
+    updateRenderables();
 }
 
 void SpeakerNode::updateAABB()
@@ -420,4 +436,70 @@ void SpeakerNode::_applyTransformation()
 	freezeTransform();
 }
 
+void SpeakerNode::onSelectionStatusChange(bool changeGroupStatus)
+{
+    EntityNode::onSelectionStatusChange(changeGroupStatus);
+
+    // Radius renderable is not always prepared for rendering, queue an update
+    updateRenderables();
+}
+
+
+void SpeakerNode::onEntitySettingsChanged()
+{
+    EntityNode::onEntitySettingsChanged();
+
+    _showRadiiWhenUnselected = EntitySettings::InstancePtr()->getShowAllSpeakerRadii();
+    updateRenderables();
+}
+
+void SpeakerNode::onInsertIntoScene(scene::IMapRootNode& root)
+{
+    EntityNode::onInsertIntoScene(root);
+
+    updateRenderables();
+}
+
+void SpeakerNode::onRemoveFromScene(scene::IMapRootNode& root)
+{
+    EntityNode::onRemoveFromScene(root);
+
+    clearRenderables();
+}
+
+void SpeakerNode::onVisibilityChanged(bool isVisibleNow)
+{
+    EntityNode::onVisibilityChanged(isVisibleNow);
+
+    if (isVisibleNow)
+    {
+        updateRenderables();
+    }
+    else
+    {
+        clearRenderables();
+    }
+}
+
+void SpeakerNode::updateRenderables()
+{
+    _renderableBox.queueUpdate();
+    _renderableRadiiWireframe.queueUpdate();
+    _renderableRadiiFill.queueUpdate();
+    _renderableRadiiFillOutline.queueUpdate();
+}
+
+void SpeakerNode::clearRenderables()
+{
+    _renderableBox.clear();
+    _renderableRadiiWireframe.clear();
+    _renderableRadiiFill.clear();
+    _renderableRadiiFillOutline.clear();
+}
+
+const Vector3& SpeakerNode::getWorldPosition() const
+{
+    return m_origin;
+}
+
 } // namespace entity
