@@ -1,5 +1,6 @@
 #pragma once
 
+#include <sigc++/trackable.h>
 #include "scene/Node.h"
 #include "RenderableTargetLines.h"
 
@@ -16,15 +17,14 @@ class EntityNode;
  * the lines from disappearing from the view when the targeting/targeted entities
  * get culled by the space partitioning system during rendering.
  */
-class TargetLineNode :
-    public scene::Node
+class TargetLineNode final :
+    public scene::Node,
+    public sigc::trackable
 {
 private:
-    AABB _aabb;
-
     EntityNode& _owner;
 
-    mutable RenderableTargetLines _targetLines;
+    RenderableTargetLines _targetLines;
 
 public:
     TargetLineNode(EntityNode& owner);
@@ -36,10 +36,20 @@ public:
 
     const AABB& localAABB() const override;
 
-    void renderSolid(RenderableCollector& collector, const VolumeTest& volumeTest) const override;
-    void renderWireframe(RenderableCollector& collector, const VolumeTest& volumeTest) const override;
+    void onPreRender(const VolumeTest& volume) override;
+    void renderHighlights(IRenderableCollector& collector, const VolumeTest& volumeTest) override;
 	std::size_t getHighlightFlags() override;
 
+    void onRenderSystemChanged();
+
+    void onInsertIntoScene(scene::IMapRootNode& root) override;
+    void onRemoveFromScene(scene::IMapRootNode& root) override;
+
+    void queueRenderableUpdate();
+
+protected:
+    void onVisibilityChanged(bool isVisibleNow) override;
+
 private:
     Vector3 getOwnerPosition() const;
 };
