@@ -312,21 +312,16 @@ bool Map::isUnnamed() const {
 
 namespace
 {
-    bool pointfileNameMatch(const std::string& candidate,
-                            const std::string& mapStem)
-    {
-        // A matching point file either has an identical stem to the map file,
-        // or the map file stem with an underscore suffix (e.g.
-        // "mapfile_portal_123_456.lin")
-        if (candidate == mapStem)
-            return true;
-        else if (candidate.rfind(mapStem + "_", 0) == 0)
-            return true;
-        else
-            return false;
-    }
+
+bool pointfileNameMatch(const std::string& candidate, const std::string& mapStem)
+{
+    // A matching point file either has an identical stem to the map file, or the map file stem
+    // with an underscore suffix (e.g. "mapfile_portal_123_456.lin")
+    return string::iequals(candidate, mapStem) || string::istarts_with(candidate, mapStem + "_");
 }
 
+} // namespace
+
 void Map::forEachPointfile(PointfileFunctor func) const
 {
     static const char* LIN_EXT = ".lin";
@@ -400,10 +395,20 @@ void Map::setEditMode(EditMode mode)
     {
         GlobalSelectionSystem().setSelectedAll(false);
         GlobalSelectionSystem().SetMode(selection::SelectionSystem::eMergeAction);
+
+        if (getRoot())
+        {
+            getRoot()->getRenderSystem()->setMergeModeEnabled(true);
+        }
     }
     else
     {
         GlobalSelectionSystem().SetMode(selection::SelectionSystem::ePrimitive);
+
+        if (getRoot())
+        {
+            getRoot()->getRenderSystem()->setMergeModeEnabled(false);
+        }
     }
 
     signal_editModeChanged().emit(_editMode);
@@ -535,6 +540,21 @@ void Map::focusViewCmd(const cmd::ArgumentList& args)
     focusViews(args[0].getVector3(), args[1].getVector3());
 }
 
+void Map::focusCameraOnSelectionCmd(const cmd::ArgumentList& args)
+{
+    if (GlobalSelectionSystem().countSelected() == 0)
+    {
+        throw cmd::ExecutionNotPossible(_("Cannot focus, selection is empty"));
+    }
+
+    // Determine the bounds of the current selection
+    const auto& workZone = GlobalSelectionSystem().getWorkZone();
+    auto originAndAngles = scene::getOriginAndAnglesToLookAtBounds(workZone.bounds);
+
+    // Set the camera and the views to the given point
+    GlobalCameraManager().focusAllCameras(originAndAngles.first, originAndAngles.second);
+}
+
 scene::INodePtr Map::findWorldspawn()
 {
 	scene::INodePtr worldspawn;
@@ -597,7 +617,7 @@ bool Map::save(const MapFormatPtr& mapFormat)
     }
 
     // Check if the map file has been modified in the meantime
-    if (_resource->fileOnDiskHasBeenModifiedSinceLastSave() && 
+    if (_resource->fileOnDiskHasBeenModifiedSinceLastSave() &&
         !radiant::FileOverwriteConfirmation::SendAndReceiveAnswer(
             fmt::format(_("The file {0} has been modified since it was last saved,\nperhaps by another application. "
                 "Do you really want to overwrite the file?"), _mapName), _("File modification detected")))
@@ -961,14 +981,15 @@ void Map::saveAutomaticMapBackup(const cmd::ArgumentList& args)
 void Map::registerCommands()
 {
     GlobalCommandSystem().addCommand("NewMap", Map::newMap);
-    GlobalCommandSystem().addCommand("OpenMap", Map::openMap, { cmd::ARGTYPE_STRING | cmd::ARGTYPE_OPTIONAL });
+    GlobalCommandSystem().addCommand("OpenMap", std::bind(&Map::openMapCmd, this, std::placeholders::_1), 
+        { cmd::ARGTYPE_STRING | cmd::ARGTYPE_OPTIONAL });
     GlobalCommandSystem().addCommand("OpenMapFromArchive", Map::openMapFromArchive, { cmd::ARGTYPE_STRING, cmd::ARGTYPE_STRING });
     GlobalCommandSystem().addCommand("ImportMap", Map::importMap);
-    GlobalCommandSystem().addCommand("StartMergeOperation", std::bind(&Map::startMergeOperationCmd, this, std::placeholders::_1), 
+    GlobalCommandSystem().addCommand("StartMergeOperation", std::bind(&Map::startMergeOperationCmd, this, std::placeholders::_1),
         { cmd::ARGTYPE_STRING | cmd::ARGTYPE_OPTIONAL, cmd::ARGTYPE_STRING | cmd::ARGTYPE_OPTIONAL });
     GlobalCommandSystem().addCommand("AbortMergeOperation", std::bind(&Map::abortMergeOperationCmd, this, std::placeholders::_1));
     GlobalCommandSystem().addCommand("FinishMergeOperation", std::bind(&Map::finishMergeOperationCmd, this, std::placeholders::_1));
-    GlobalCommandSystem().addCommand(LOAD_PREFAB_AT_CMD, std::bind(&Map::loadPrefabAt, this, std::placeholders::_1), 
+    GlobalCommandSystem().addCommand(LOAD_PREFAB_AT_CMD, std::bind(&Map::loadPrefabAt, this, std::placeholders::_1),
         { cmd::ARGTYPE_STRING, cmd::ARGTYPE_VECTOR3, cmd::ARGTYPE_INT|cmd::ARGTYPE_OPTIONAL, cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL });
     GlobalCommandSystem().addCommand("SaveSelectedAsPrefab", Map::saveSelectedAsPrefab);
     GlobalCommandSystem().addCommand("SaveMap", std::bind(&Map::saveMapCmd, this, std::placeholders::_1));
@@ -980,6 +1001,7 @@ void Map::registerCommands()
     GlobalCommandSystem().addCommand("SaveSelected", Map::exportSelection);
 	GlobalCommandSystem().addCommand("ReloadSkins", map::algorithm::reloadSkins);
     GlobalCommandSystem().addCommand("FocusViews", std::bind(&Map::focusViewCmd, this, std::placeholders::_1), { cmd::ARGTYPE_VECTOR3, cmd::ARGTYPE_VECTOR3 });
+    GlobalCommandSystem().addCommand("FocusCameraOnSelection", std::bind(&Map::focusCameraOnSelectionCmd, this, std::placeholders::_1));
 	GlobalCommandSystem().addCommand("ExportSelectedAsModel", map::algorithm::exportSelectedAsModelCmd,
         { cmd::ARGTYPE_STRING,
           cmd::ARGTYPE_STRING,
@@ -1028,9 +1050,9 @@ void Map::newMap(const cmd::ArgumentList& args)
     }
 }
 
-void Map::openMap(const cmd::ArgumentList& args)
+void Map::openMapCmd(const cmd::ArgumentList& args)
 {
-    if (!GlobalMap().askForSave(_("Open Map"))) return;
+    if (!askForSave(_("Open Map"))) return;
 
     std::string candidate;
 
@@ -1041,7 +1063,7 @@ void Map::openMap(const cmd::ArgumentList& args)
     else
     {
         // No arguments passed, get the map file name to load
-        MapFileSelection fileInfo = MapFileManager::getMapFileSelection(true, _("Open map"), filetype::TYPE_MAP);
+        auto fileInfo = MapFileManager::getMapFileSelection(true, _("Open map"), filetype::TYPE_MAP);
         candidate = fileInfo.fullPath;
     }
 
@@ -1080,8 +1102,8 @@ void Map::openMap(const cmd::ArgumentList& args)
 	{
         GlobalMRU().insert(mapToLoad);
 
-        GlobalMap().freeMap();
-        GlobalMap().load(mapToLoad);
+        freeMap();
+        load(mapToLoad);
     }
 }
 
