@@ -15,7 +15,7 @@ namespace model
 	class StaticModelSurface;
 	typedef std::shared_ptr<StaticModelSurface> StaticModelSurfacePtr;
 }
-class RenderableCollector;
+class IRenderableCollector;
 class RendererLight;
 class SelectionTest;
 class Selector;
@@ -63,7 +63,7 @@ private:
 	typedef std::vector<Surface> SurfaceList;
 
 	// Vector of renderable surfaces for this model
-	SurfaceList _surfVec;
+	SurfaceList _surfaces;
 
 	// The current working scale
 	Vector3 _scaleTransformed;
@@ -89,6 +89,9 @@ private:
 	// Undoable stuff
 	IUndoStateSaver* _undoStateSaver;
 
+    sigc::signal<void> _sigShadersChanged;
+    sigc::signal<void> _sigSurfaceScaleApplied;
+
 private:
 
 	// Update the list of materials by querying each surface for its current
@@ -120,28 +123,20 @@ public:
 	void connectUndoSystem(IUndoSystem& undoSystem);
 	void disconnectUndoSystem(IUndoSystem& undoSystem);
 
-    // Delegated render methods called by StaticModelNode (not part of any
-    // interface)
-	void renderSolid(RenderableCollector& rend, const Matrix4& localToWorld,
-                     const IRenderEntity& entity, const LitObject& litObj) const;
-	void renderWireframe(RenderableCollector& rend, const Matrix4& localToWorld,
-		const IRenderEntity& entity) const;
-
 	void setRenderSystem(const RenderSystemPtr& renderSystem);
 
-	/**
-	 * Back-end render function from OpenGLRenderable. This is called from the
-	 * model selector but not the main renderer, which uses the front-end render
-	 * method.
-	 */
-	void render(const RenderInfo& info) const override;
+    // A signal that is emitted after the captured shaders have been changed (or cleared)
+    sigc::signal<void>& signal_ShadersChanged();
+
+    // Signal emitted when any surface scale has been changed
+    sigc::signal<void>& signal_SurfaceScaleApplied();
 
 	/**
 	 * Return the number of surfaces in this model.
 	 */
 	int getSurfaceCount() const override
     {
-		return static_cast<int>(_surfVec.size());
+		return static_cast<int>(_surfaces.size());
 	}
 
 	/**
@@ -211,8 +206,8 @@ public:
 	 */
 	const SurfaceList& getSurfaces() const;
 
-	// Revert to base scale
-	void revertScale();
+	// Revert to base scale (returns true if the scale was actually changed by this call)
+	bool revertScale();
 
 	// TransformationChanged, apply the given scale to the "working copy"
 	void evaluateScale(const Vector3& scale);
@@ -226,6 +221,8 @@ public:
 
 	// Returns the current base scale of this model
 	const Vector3& getScale() const;
+
+    void foreachSurface(const std::function<void(const StaticModelSurface&)>& func) const;
 };
 typedef std::shared_ptr<StaticModel> StaticModelPtr;
 
