@@ -7,18 +7,19 @@
 #include "math/Frustum.h"
 #include "math/Hash.h"
 
-// Construct a PatchNode with no arguments
 PatchNode::PatchNode(patch::PatchDefType type) :
 	scene::SelectableNode(),
 	m_dragPlanes(std::bind(&PatchNode::selectedChangedComponent, this, std::placeholders::_1)),
-	m_render_selected(GL_POINTS),
 	m_patch(*this),
-    _untransformedOriginChanged(true)
+    _untransformedOriginChanged(true),
+    _renderableSurfaceSolid(m_patch.getTesselation(), true),
+    _renderableSurfaceWireframe(m_patch.getTesselation(), false),
+    _renderableCtrlLattice(m_patch, m_ctrl_instances),
+    _renderableCtrlPoints(m_patch, m_ctrl_instances)
 {
 	m_patch.setFixedSubdivisions(type == patch::PatchDefType::Def3, Subdivisions(m_patch.getSubdivisions()));
 }
 
-// Copy Constructor
 PatchNode::PatchNode(const PatchNode& other) :
 	scene::SelectableNode(other),
 	scene::Cloneable(other),
@@ -29,18 +30,17 @@ PatchNode::PatchNode(const PatchNode& other) :
 	ComponentEditable(other),
 	ComponentSnappable(other),
 	PlaneSelectable(other),
-	LitObject(other),
 	Transformable(other),
 	m_dragPlanes(std::bind(&PatchNode::selectedChangedComponent, this, std::placeholders::_1)),
-	m_render_selected(GL_POINTS),
 	m_patch(other.m_patch, *this), // create the patch out of the <other> one
-    _untransformedOriginChanged(true)
+    _untransformedOriginChanged(true),
+    _renderableSurfaceSolid(m_patch.getTesselation(), true),
+    _renderableSurfaceWireframe(m_patch.getTesselation(), false),
+    _renderableCtrlLattice(m_patch, m_ctrl_instances),
+    _renderableCtrlPoints(m_patch, m_ctrl_instances)
 {
 }
 
-PatchNode::~PatchNode()
-{}
-
 scene::INode::Type PatchNode::getNodeType() const
 {
 	return Type::Patch;
@@ -82,19 +82,23 @@ std::string PatchNode::getFingerprint()
     return hash;
 }
 
-void PatchNode::allocate(std::size_t size) {
+void PatchNode::updateSelectableControls()
+{
 	// Clear the control instance vector and reserve <size> memory
 	m_ctrl_instances.clear();
-	m_ctrl_instances.reserve(size);
+
+    // We link the instances to the working control point set
+    auto& ctrlPoints = m_patch.getControlPointsTransformed();
+
+	m_ctrl_instances.reserve(ctrlPoints.size());
 
 	// greebo: Cycle through the patch's control vertices and add them as PatchControlInstance to the vector
 	// The PatchControlInstance constructor takes a pointer to a PatchControl and the SelectionChanged callback
 	// The passed callback points back to this class (the member method selectedChangedComponent() is called).
-	for(PatchControlIter i = m_patch.begin(); i != m_patch.end(); ++i)
+	for(auto& ctrl : ctrlPoints)
 	{
-		m_ctrl_instances.push_back(
-			PatchControlInstance(*i, std::bind(&PatchNode::selectedChangedComponent, this, std::placeholders::_1))
-		);
+		m_ctrl_instances.emplace_back(ctrl, 
+            std::bind(&PatchNode::selectedChangedComponent, this, std::placeholders::_1));
 	}
 }
 
@@ -129,21 +133,24 @@ bool PatchNode::selectedVertices() {
     return false;
 }
 
-void PatchNode::snapComponents(float snap) {
+void PatchNode::snapComponents(float snap)
+{
 	// Are there any selected vertices
-	if (selectedVertices()) {
-		// Tell the patch to save the current undo state
-		m_patch.undoSave();
+    if (!selectedVertices()) return;
 
-		// Cycle through all the selected control instances and snap them to the grid
-		for (PatchControlInstances::iterator i = m_ctrl_instances.begin(); i != m_ctrl_instances.end(); ++i) {
-			if(i->isSelected()) {
-				i->snapto(snap);
-			}
-		}
-		// Tell the patch that control points have changed
-		m_patch.controlPointsChanged();
-	}
+    // Cycle through all the selected control instances and snap them to the grid
+    for (auto& vertex : m_ctrl_instances)
+    {
+	    if (vertex.isSelected())
+        {
+            vertex.snapto(snap);
+	    }
+    }
+
+    // Save the transformed control point array to the real set
+    m_patch.freezeTransform();
+    // Tell the patch that control points have changed
+    m_patch.controlPointsChanged();
 }
 
 // Test the Patch instance for selection
@@ -211,15 +218,18 @@ void PatchNode::invertSelectedComponents(selection::ComponentSelectionMode mode)
 	}
 }
 
-void PatchNode::testSelectComponents(Selector& selector, SelectionTest& test, selection::ComponentSelectionMode mode) {
+void PatchNode::testSelectComponents(Selector& selector, SelectionTest& test, selection::ComponentSelectionMode mode)
+{
 	test.BeginMesh(localToWorld());
 
 	// Only react to eVertex selection mode
 	switch(mode) {
-        case selection::ComponentSelectionMode::Vertex: {
+        case selection::ComponentSelectionMode::Vertex:
+        {
 			// Cycle through all the control instances and test them for selection
-			for (PatchControlInstances::iterator i = m_ctrl_instances.begin(); i != m_ctrl_instances.end(); ++i) {
-				i->testSelect(selector, test);
+			for (auto& i : m_ctrl_instances)
+            {
+				i.testSelect(selector, test);
 			}
 		}
 		break;
@@ -252,7 +262,11 @@ bool PatchNode::hasVisibleMaterial() const
 	return m_patch.getSurfaceShader().getGLShader()->getMaterial()->isVisible();
 }
 
-void PatchNode::selectedChangedComponent(const ISelectable& selectable) {
+void PatchNode::selectedChangedComponent(const ISelectable& selectable)
+{
+    // We need to update our vertex colours next time we render them
+    _renderableCtrlPoints.queueUpdate();
+
 	// Notify the selection system that this PatchNode was selected. The RadiantSelectionSystem adds
 	// this to its internal list of selected nodes.
 	GlobalSelectionSystem().onComponentSelection(SelectableNode::getSelf(), selectable);
@@ -264,11 +278,41 @@ scene::INodePtr PatchNode::clone() const
 	return std::make_shared<PatchNode>(*this);
 }
 
+void PatchNode::updateAllRenderables()
+{
+    _renderableSurfaceSolid.queueUpdate();
+    _renderableSurfaceWireframe.queueUpdate();
+    _renderableCtrlLattice.queueUpdate();
+    _renderableCtrlPoints.queueUpdate();
+}
+
+void PatchNode::hideAllRenderables()
+{
+    _renderableSurfaceSolid.hide();
+    _renderableSurfaceWireframe.hide();
+    _renderableCtrlLattice.hide();
+    _renderableCtrlPoints.hide();
+}
+
+void PatchNode::clearAllRenderables()
+{
+    _renderableSurfaceSolid.clear();
+    _renderableSurfaceWireframe.clear();
+    _renderableCtrlLattice.clear();
+    _renderableCtrlPoints.clear();
+}
+
 void PatchNode::onInsertIntoScene(scene::IMapRootNode& root)
 {
     // Mark the GL shader as used from now on, this is used by the TextureBrowser's filtering
     m_patch.getSurfaceShader().setInUse(true);
 
+    // When inserting a patch into the scene, it gets a parent entity assigned
+    // The colour of that entity will influence the tesselation's vertex colours
+    m_patch.queueTesselationUpdate();
+
+    updateAllRenderables();
+
 	m_patch.connectUndoSystem(root.getUndoSystem());
 	GlobalCounters().getCounter(counterPatches).increment();
 
@@ -290,6 +334,8 @@ void PatchNode::onRemoveFromScene(scene::IMapRootNode& root)
 
 	m_patch.disconnectUndoSystem(root.getUndoSystem());
 
+    clearAllRenderables();
+
     m_patch.getSurfaceShader().setInUse(false);
 
 	SelectableNode::onRemoveFromScene(root);
@@ -300,47 +346,48 @@ bool PatchNode::getIntersection(const Ray& ray, Vector3& intersection)
 	return m_patch.getIntersection(ray, intersection);
 }
 
-bool PatchNode::intersectsLight(const RendererLight& light) const {
-	return light.lightAABB().intersects(worldAABB());
-}
-
-void PatchNode::renderSolid(RenderableCollector& collector, const VolumeTest& volume) const
+void PatchNode::onPreRender(const VolumeTest& volume)
 {
-	// Don't render invisible shaders
-	if (!isForcedVisible() && !m_patch.hasVisibleMaterial()) return;
-
     // Defer the tesselation calculation to the last minute
-	const_cast<Patch&>(m_patch).evaluateTransform();
-    const_cast<Patch&>(m_patch).updateTesselation();
+    m_patch.evaluateTransform();
+    m_patch.updateTesselation();
 
-	assert(_renderEntity); // patches rendered without parent - no way!
+    _renderableSurfaceSolid.update(m_patch._shader.getGLShader());
+    _renderableSurfaceWireframe.update(_renderEntity->getWireShader());
 
-    // Render the patch itself
-    collector.addRenderable(
-        *m_patch._shader.getGLShader(), m_patch._solidRenderable,
-        localToWorld(), this, _renderEntity
-    );
+    _renderableSurfaceSolid.attachToEntity(_renderEntity);
 
-#if DEBUG_PATCH_NTB_VECTORS
-    m_patch._renderableVectors.render(collector, volume, localToWorld());
-#endif
+    if (isSelected() && GlobalSelectionSystem().ComponentMode() == selection::ComponentSelectionMode::Vertex)
+    {
+        // Selected patches in component mode render the lattice connecting the control points
+        _renderableCtrlLattice.update(_ctrlLatticeShader);
+        _renderableCtrlPoints.update(_ctrlPointShader);
+    }
+    else
+    {
+        _renderableCtrlPoints.hide();
+        _renderableCtrlLattice.hide();
 
-	// Render the selected components
-	renderComponentsSelected(collector, volume);
+        // Queue an update the next time it's rendered
+        _renderableCtrlPoints.queueUpdate();
+        _renderableCtrlLattice.queueUpdate();
+    }
 }
 
-void PatchNode::renderWireframe(RenderableCollector& collector, const VolumeTest& volume) const
+void PatchNode::renderHighlights(IRenderableCollector& collector, const VolumeTest& volume)
 {
-	// Don't render invisible shaders
-	if (!isForcedVisible() && !m_patch.hasVisibleMaterial()) return;
+    if (GlobalSelectionSystem().Mode() != selection::SelectionSystem::eComponent)
+    {
+        // The coloured selection overlay should use the same triangulated surface to avoid z fighting
+        collector.setHighlightFlag(IRenderableCollector::Highlight::Faces, true);
+        collector.setHighlightFlag(IRenderableCollector::Highlight::Primitives, false);
+        collector.addHighlightRenderable(_renderableSurfaceSolid, localToWorld());
+    }
 
-	const_cast<Patch&>(m_patch).evaluateTransform();
-
-	// Pass the call to the patch instance, it adds the renderable
-	m_patch.renderWireframe(collector, volume, localToWorld(), *_renderEntity);
-
-	// Render the selected components
-	renderComponentsSelected(collector, volume);
+    // The selection outline (wireframe) should use the quadrangulated surface
+    collector.setHighlightFlag(IRenderableCollector::Highlight::Faces, false);
+    collector.setHighlightFlag(IRenderableCollector::Highlight::Primitives, true);
+    collector.addHighlightRenderable(_renderableSurfaceWireframe, localToWorld());
 }
 
 void PatchNode::setRenderSystem(const RenderSystemPtr& renderSystem)
@@ -349,63 +396,17 @@ void PatchNode::setRenderSystem(const RenderSystemPtr& renderSystem)
 
 	m_patch.setRenderSystem(renderSystem);
 
-	if (renderSystem)
+    clearAllRenderables();
+
+    if (renderSystem)
 	{
-		m_state_selpoint = renderSystem->capture("$SELPOINT");
+        _ctrlPointShader = renderSystem->capture(BuiltInShaderType::BigPoint);
+        _ctrlLatticeShader = renderSystem->capture(BuiltInShaderType::PatchLattice);
 	}
 	else
 	{
-		m_state_selpoint.reset();
-	}
-}
-
-// Renders the components of this patch instance
-void PatchNode::renderComponents(RenderableCollector& collector, const VolumeTest& volume) const
-{
-	// Don't render invisible shaders
-	if (!m_patch.getSurfaceShader().getGLShader()->getMaterial()->isVisible()) return;
-
-	// greebo: Don't know yet, what evaluateTransform() is really doing
-	const_cast<Patch&>(m_patch).evaluateTransform();
-
-	// Only render the components, if we are in the according ComponentMode
-	if (GlobalSelectionSystem().ComponentMode() == selection::ComponentSelectionMode::Vertex)
-    {
-		m_patch.submitRenderablePoints(collector, volume, localToWorld());
-	}
-}
-
-void PatchNode::update_selected() const {
-	// Clear the renderable point vector that represents the selection
-	m_render_selected.clear();
-
-	// Cycle through the transformed patch vertices and set the colour of all selected control vertices to BLUE (hardcoded)
-	PatchControlConstIter ctrl = m_patch.getControlPointsTransformed().begin();
-
-	for (PatchControlInstances::const_iterator i = m_ctrl_instances.begin();
-		 i != m_ctrl_instances.end(); ++i, ++ctrl)
-	{
-		if (i->isSelected()) {
-			const Colour4b colour_selected(0, 0, 0, 255);
-			// Add this patch control instance to the render list
-			m_render_selected.push_back(VertexCb(reinterpret_cast<const Vertex3f&>(ctrl->vertex), colour_selected));
-		}
-	}
-}
-
-void PatchNode::renderComponentsSelected(RenderableCollector& collector, const VolumeTest& volume) const
-{
-	// greebo: Don't know yet, what evaluateTransform() is really doing
-	const_cast<Patch&>(m_patch).evaluateTransform();
-
-	// Rebuild the array of selected control vertices
-	update_selected();
-
-	// If there are any selected components, add them to the collector
-	if (!m_render_selected.empty())
-    {
-		collector.setHighlightFlag(RenderableCollector::Highlight::Primitives, false);
-		collector.addRenderable(*m_state_selpoint, m_render_selected, localToWorld());
+        _ctrlPointShader.reset();
+        _ctrlLatticeShader.reset();
 	}
 }
 
@@ -467,6 +468,8 @@ void PatchNode::transformComponents(const Matrix4& matrix) {
 void PatchNode::_onTransformationChanged()
 {
 	m_patch.transformChanged();
+
+    updateAllRenderables();
 }
 
 void PatchNode::_applyTransformation()
@@ -489,3 +492,34 @@ const Vector3& PatchNode::getUntransformedOrigin()
 
     return _untransformedOrigin;
 }
+
+void PatchNode::onTesselationChanged()
+{
+    updateAllRenderables();
+}
+
+void PatchNode::onControlPointsChanged()
+{
+    updateAllRenderables();
+}
+
+void PatchNode::onMaterialChanged()
+{
+    _renderableSurfaceSolid.queueUpdate();
+    _renderableSurfaceWireframe.queueUpdate();
+}
+
+void PatchNode::onVisibilityChanged(bool visible)
+{
+    SelectableNode::onVisibilityChanged(visible);
+
+    if (!visible)
+    {
+        hideAllRenderables();
+    }
+    else
+    {
+        // Queue an update, renderables are automatically shown in onPreRender
+        updateAllRenderables();
+    }
+}
