@@ -9,8 +9,9 @@
 #include "scene/SelectableNode.h"
 #include "PatchControlInstance.h"
 #include "dragplanes.h"
+#include "PatchRenderables.h"
 
-class PatchNode :
+class PatchNode final :
 	public scene::SelectableNode,
 	public scene::Cloneable,
 	public Snappable,
@@ -21,7 +22,6 @@ class PatchNode :
 	public ComponentEditable,
 	public ComponentSnappable,
 	public PlaneSelectable,
-	public LitObject,
 	public Transformable,
 	public ITraceable,
     public scene::IComparableNode
@@ -32,33 +32,30 @@ class PatchNode :
 	typedef std::vector<PatchControlInstance> PatchControlInstances;
 	PatchControlInstances m_ctrl_instances;
 
-	// An array of renderable points
-	mutable RenderablePointVector m_render_selected;
-
 	Patch m_patch;
 
 	// An internal AABB variable to calculate the bounding box of the selected components (has to be mutable)
 	mutable AABB m_aabb_component;
 
-	ShaderPtr m_state_selpoint;
+	ShaderPtr _ctrlPointShader;
+    ShaderPtr _ctrlLatticeShader;
 
     // For pivoted rotations, we need a copy of this lying around
     Vector3 _untransformedOrigin;
     // If true, the _untransformedOrigin member needs an update
     bool _untransformedOriginChanged;
 
+    RenderablePatchTesselation<TesselationIndexer_Triangles> _renderableSurfaceSolid;
+    RenderablePatchTesselation<TesselationIndexer_Quads> _renderableSurfaceWireframe;
+    RenderablePatchLattice _renderableCtrlLattice; // Wireframe connecting the control points
+    RenderablePatchControlPoints _renderableCtrlPoints; // the coloured control points
+
 public:
-	// Construct a PatchNode with no arguments
 	PatchNode(patch::PatchDefType type);
 
 	// Copy Constructor
 	PatchNode(const PatchNode& other);
 
-	virtual ~PatchNode();
-
-	// Patch::Observer implementation
-	void allocate(std::size_t size);
-
 	std::string name() const override;
 	Type getNodeType() const override;
 
@@ -92,8 +89,8 @@ public:
 	void testSelectComponents(Selector& selector, SelectionTest& test, selection::ComponentSelectionMode mode) override;
 
 	// override scene::Inode::onRemoveFromScene to deselect the child components
-    virtual void onInsertIntoScene(scene::IMapRootNode& root) override;
-    virtual void onRemoveFromScene(scene::IMapRootNode& root) override;
+    void onInsertIntoScene(scene::IMapRootNode& root) override;
+    void onRemoveFromScene(scene::IMapRootNode& root) override;
 
 	// Traceable implementation
 	bool getIntersection(const Ray& ray, Vector3& intersection) override;
@@ -123,26 +120,25 @@ public:
 	// (see ObservedSelectable and PatchControlInstance)
 	void selectedChangedComponent(const ISelectable& selectable);
 
-	// LitObject implementation
-	bool intersectsLight(const RendererLight& light) const override;
-
 	// Renderable implementation
 
 	// Render functions, these make sure that all things get rendered properly. The calls are also passed on
 	// to the contained patch <m_patch>
-	void renderSolid(RenderableCollector& collector, const VolumeTest& volume) const override;
-	void renderWireframe(RenderableCollector& collector, const VolumeTest& volume) const override;
+    void onPreRender(const VolumeTest& volume) override;
+	void renderHighlights(IRenderableCollector& collector, const VolumeTest& volume) override;
 	void setRenderSystem(const RenderSystemPtr& renderSystem) override;
 
-	// Renders the components of this patch instance, makes use of the Patch::render_component() method
-	void renderComponents(RenderableCollector& collector, const VolumeTest& volume) const override;
-
 	void evaluateTransform();
 	std::size_t getHighlightFlags() override;
 
     // Returns the center of the untransformed world AABB
     const Vector3& getUntransformedOrigin() override;
 
+    void onControlPointsChanged();
+    void onMaterialChanged();
+    void onTesselationChanged();
+    void updateSelectableControls();
+
 protected:
 	// Gets called by the Transformable implementation whenever
 	// scale, rotation or translation is changed.
@@ -152,16 +148,14 @@ protected:
 	// or when reverting transformations.
     void _applyTransformation() override;
 
+    void onVisibilityChanged(bool isVisibleNow) override;
+
 private:
 	// Transforms the patch components with the given transformation matrix
 	void transformComponents(const Matrix4& matrix);
 
-	// greebo: Updates the internal render array m_render_selected, that contains all control vertices that should be
-	// rendered as highlighted.
-	void update_selected() const;
-
-	// greebo: Renders the selected components. This is called by the above two render functions
-	void renderComponentsSelected(RenderableCollector& collector, const VolumeTest& volume) const;
+    void updateAllRenderables();
+    void hideAllRenderables();
+    void clearAllRenderables();
 };
 typedef std::shared_ptr<PatchNode> PatchNodePtr;
-typedef std::weak_ptr<PatchNode> PatchNodeWeakPtr;
