@@ -1,9 +1,11 @@
 #include "GLProgramFactory.h"
 
-#include "glprogram/GLSLDepthFillProgram.h"
-#include "glprogram/GLSLDepthFillAlphaProgram.h"
-#include "glprogram/GLSLBumpProgram.h"
+#include "glprogram/DepthFillAlphaProgram.h"
+#include "glprogram/CubeMapProgram.h"
+#include "glprogram/InteractionProgram.h"
 #include "glprogram/GenericVFPProgram.h"
+#include "glprogram/ShadowMapProgram.h"
+#include "glprogram/RegularStageProgram.h"
 
 #include "itextstream.h"
 #include "iregistry.h"
@@ -18,25 +20,29 @@
 namespace render
 {
 
+using CharBufPtr = std::shared_ptr<std::vector<char>>;
+
 // Constructor, populates map with GLProgram instances
 GLProgramFactory::GLProgramFactory()
 {
-    _builtInPrograms["depthFill"] = std::make_shared<GLSLDepthFillProgram>();
-    _builtInPrograms["depthFillAlpha"] = std::make_shared<GLSLDepthFillAlphaProgram>();
-    _builtInPrograms["bumpMap"] = std::make_shared<GLSLBumpProgram>();
+    _builtInPrograms[ShaderProgram::DepthFillAlpha] = std::make_shared<DepthFillAlphaProgram>();
+    _builtInPrograms[ShaderProgram::Interaction] = std::make_shared<InteractionProgram>();
+    _builtInPrograms[ShaderProgram::CubeMap] = std::make_shared<CubeMapProgram>();
+    _builtInPrograms[ShaderProgram::ShadowMap] = std::make_shared<ShadowMapProgram>();
+    _builtInPrograms[ShaderProgram::RegularStage] = std::make_shared<RegularStageProgram>();
 }
 
-GLProgram* GLProgramFactory::getBuiltInProgram(const std::string& name)
+GLProgram* GLProgramFactory::getBuiltInProgram(ShaderProgram builtInProgram)
 {
 	// Lookup the program, if not found throw an exception
-	ProgramMap::iterator i = _builtInPrograms.find(name);
+	auto i = _builtInPrograms.find(builtInProgram);
 
     if (i != _builtInPrograms.end())
     {
         return i->second.get();
     }
 
-	throw std::runtime_error("GLProgramFactory: failed to find program " + name);
+	throw std::runtime_error("GLProgramFactory: failed to find program " + string::to_string((int)builtInProgram));
 }
 
 GLProgram* GLProgramFactory::getProgram(const std::string& vertexProgramFilename,
@@ -70,7 +76,7 @@ void GLProgramFactory::realise()
 }
 
 // Unrealise the program factory.
-void GLProgramFactory::unrealise() 
+void GLProgramFactory::unrealise()
 {
 	// Destroy each GLProgram in the map
     for (ProgramMap::value_type& pair : _builtInPrograms)
@@ -79,37 +85,43 @@ void GLProgramFactory::unrealise()
 	}
 }
 
+namespace
+{
+
+// Get the path of a GL program file
+std::string getBuiltInGLProgramPath(const std::string& progName)
+{
+    // Append the requested filename with the "gl/" directory.
+    return module::GlobalModuleRegistry().getApplicationContext().getRuntimeDataPath()
+         + "gl/" + progName;
+}
+
 // Get file as a char buffer
-GLProgramFactory::CharBufPtr
-GLProgramFactory::getFileAsBuffer(const std::string& filename,
-                                  bool nullTerminated)
+CharBufPtr getFileAsBuffer(const std::string& filename)
 {
     // Get absolute path from filename
     std::string absFileName = getBuiltInGLProgramPath(filename);
 
     // Open the file
-	std::size_t size = os::getFileSize(absFileName);
-	std::ifstream file(absFileName.c_str());
+    std::size_t size = os::getFileSize(absFileName);
+    std::ifstream file(absFileName.c_str());
 
     // Throw an exception if the file could not be found
-	if (!file.is_open())
-    {
-        throw std::runtime_error(
-            "GLProgramFactory: failed to open file: " + absFileName
-        );
+    if (!file.is_open()) {
+        throw std::runtime_error("GLProgramFactory: failed to open file: " + absFileName);
     }
 
     // Read the file data into a buffer, adding a NULL terminator if required
-    std::size_t bufSize = (nullTerminated ? size + 1 : size);
-	CharBufPtr buffer(new std::vector<char>(bufSize, 0));
-	file.read(&buffer->front(), size);
+    std::size_t bufSize = size + 1;
+    CharBufPtr buffer(new std::vector<char>(bufSize, 0));
+    file.read(&buffer->front(), size);
 
     // Close file and return buffer
     file.close();
     return buffer;
 }
 
-void GLProgramFactory::assertShaderCompiled(GLuint shader)
+void assertShaderCompiled(GLuint shader, const std::string& filename)
 {
     // Get compile status
     GLint compileStatus;
@@ -128,13 +140,13 @@ void GLProgramFactory::assertShaderCompiled(GLuint shader)
         // Convert to string and throw exception
         std::string logStr = std::string(&logBuf.front());
         throw std::runtime_error(
-            "Failed to compile GLSL shader:\n"
+            "Failed to compile GLSL shader \"" + filename + "\":\n"
             + logStr
         );
     }
 }
 
-std::string GLProgramFactory::getProgramInfoLog(GLuint program)
+std::string getProgramInfoLog(GLuint program)
 {
     // Get log length
     int logLength;
@@ -149,7 +161,7 @@ std::string GLProgramFactory::getProgramInfoLog(GLuint program)
     return logStr;
 }
 
-void GLProgramFactory::assertProgramLinked(GLuint program)
+void assertProgramLinked(GLuint program)
 {
     // Check the link status
     GLint linkStatus;
@@ -181,6 +193,8 @@ void GLProgramFactory::assertProgramLinked(GLuint program)
 #endif
 }
 
+} // namespace
+
 GLuint GLProgramFactory::createGLSLProgram(const std::string& vFile,
                                            const std::string& fFile)
 {
@@ -193,8 +207,8 @@ GLuint GLProgramFactory::createGLSLProgram(const std::string& vFile,
 
     // Load the source files as NULL-terminated strings and pass the text to
     // OpenGL
-    CharBufPtr vertexSrc = getFileAsBuffer(vFile, true);
-    CharBufPtr fragSrc = getFileAsBuffer(fFile, true);
+    CharBufPtr vertexSrc = getFileAsBuffer(vFile);
+    CharBufPtr fragSrc = getFileAsBuffer(fFile);
 
     const char* csVertex = &vertexSrc->front();
     const char* csFragment = &fragSrc->front();
@@ -205,10 +219,10 @@ GLuint GLProgramFactory::createGLSLProgram(const std::string& vFile,
 
     // Compile the shaders
     glCompileShader(vertexShader);
-    assertShaderCompiled(vertexShader);
+    assertShaderCompiled(vertexShader, vFile);
 
     glCompileShader(fragmentShader);
-    assertShaderCompiled(fragmentShader);
+    assertShaderCompiled(fragmentShader, fFile);
 
     debug::assertNoGlErrors();
 
@@ -226,14 +240,4 @@ GLuint GLProgramFactory::createGLSLProgram(const std::string& vFile,
     return program;
 }
 
-// Get the path of a GL program file
-std::string GLProgramFactory::getBuiltInGLProgramPath(const std::string& progName)
-{
-    // Append the requested filename with the "gl/" directory.
-    return module::GlobalModuleRegistry()
-            .getApplicationContext()
-                .getRuntimeDataPath()
-                    + "gl/" + progName;
-}
-
 } // namespace render
