@@ -16,12 +16,14 @@
 #include "messages/FileOverwriteConfirmation.h"
 #include "messages/MapFileOperation.h"
 #include "messages/FileSaveConfirmation.h"
+#include "messages/NotificationMessage.h"
 #include "algorithm/Scene.h"
 #include "algorithm/XmlUtils.h"
 #include "algorithm/Primitives.h"
 #include "os/file.h"
 #include <sigc++/connection.h>
 #include "testutil/FileSelectionHelper.h"
+#include "registry/registry.h"
 
 using namespace std::chrono_literals;
 
@@ -1277,6 +1279,69 @@ TEST_F(MapSavingTest, AutoSaverDoesntChangeSaveCopyAsFilename)
     GlobalRadiantCore().getMessageBus().removeListener(msgSubscription);
 }
 
+TEST_F(MapSavingTest, AutoSaveSnapshotsSupportRelativePaths)
+{
+    std::string modRelativePath = "maps/altar.map";
+    GlobalCommandSystem().executeCommand("OpenMap", modRelativePath);
+    checkAltarScene();
+
+    // Set this to a relative path
+    registry::setValue(map::RKEY_AUTOSAVE_SNAPSHOTS_ENABLED, true);
+    registry::setValue(map::RKEY_AUTOSAVE_SNAPSHOTS_FOLDER, "customsnapshots/");
+
+    // We expect the file to end up here
+    std::string expectedSnapshotPath = "maps/customsnapshots/altar.0.map";
+
+    EXPECT_FALSE(GlobalFileSystem().openTextFile(expectedSnapshotPath)) << "Snapshot already exists in " << expectedSnapshotPath;
+
+    // Trigger an auto save now
+    GlobalAutoSaver().performAutosave();
+
+    EXPECT_TRUE(GlobalFileSystem().openTextFile(expectedSnapshotPath)) << "Snapshot should now exist in " << expectedSnapshotPath;
+    
+    // Load and confirm the saved scene
+    GlobalCommandSystem().executeCommand("OpenMap", expectedSnapshotPath);
+    checkAltarScene();
+
+    auto fullPath = GlobalFileSystem().findFile(expectedSnapshotPath) + expectedSnapshotPath;
+    EXPECT_NE(fullPath, "") << "Failed to resolve absolute file path of " << expectedSnapshotPath;
+
+    if (!fullPath.empty())
+    {
+        fs::remove(os::replaceExtension(fullPath, "darkradiant"));
+        fs::remove(fullPath);
+    }
+}
+
+TEST_F(MapSavingTest, AutoSaveSnapshotsSupportAbsolutePaths)
+{
+    std::string modRelativePath = "maps/altar.map";
+    GlobalCommandSystem().executeCommand("OpenMap", modRelativePath);
+    checkAltarScene();
+
+    // Set this to an absolute path
+    auto snapshotFolder = _context.getTemporaryDataPath() + "customsnapshots/";
+    registry::setValue(map::RKEY_AUTOSAVE_SNAPSHOTS_ENABLED, true);
+    registry::setValue(map::RKEY_AUTOSAVE_SNAPSHOTS_FOLDER, snapshotFolder);
+
+    // We expect the file to end up there
+    std::string expectedSnapshotPath = snapshotFolder + "altar.0.map";
+
+    EXPECT_FALSE(GlobalFileSystem().openTextFileInAbsolutePath(expectedSnapshotPath)) << "Snapshot already exists in " << expectedSnapshotPath;
+
+    // Trigger an auto save now
+    GlobalAutoSaver().performAutosave();
+
+    EXPECT_TRUE(GlobalFileSystem().openTextFileInAbsolutePath(expectedSnapshotPath)) << "Snapshot should now exist in " << expectedSnapshotPath;
+
+    // Load and confirm the saved scene
+    GlobalCommandSystem().executeCommand("OpenMap", expectedSnapshotPath);
+    checkAltarScene();
+
+    fs::remove(os::replaceExtension(expectedSnapshotPath, "darkradiant"));
+    fs::remove(expectedSnapshotPath);
+}
+
 namespace
 {
 
@@ -1541,4 +1606,55 @@ TEST_F(MapSavingTest, RadiantShutdownCancelWithUnsavedChanges)
     });
 }
 
+// Listens for a warning message when loading maps without info file
+class WarningReceiver final
+{
+private:
+    std::size_t _msgSubscription;
+    std::string _receivedMessage;
+
+public:
+    WarningReceiver()
+    {
+        // Subscribe to the event asking for the target path
+        _msgSubscription = GlobalRadiantCore().getMessageBus().addListener(
+            radiant::IMessage::Type::Notification,
+            radiant::TypeListener<radiant::NotificationMessage>(
+                [this](radiant::NotificationMessage& msg)
+                {
+                    _receivedMessage = msg.getMessage();
+                }));
+    }
+
+    std::string getReceivedMessage() const
+    {
+        return _receivedMessage;
+    }
+
+    ~WarningReceiver()
+    {
+        GlobalRadiantCore().getMessageBus().removeListener(_msgSubscription);
+    }
+};
+
+TEST_F(MapLoadingTest, WarningAboutMissingInfoFile)
+{
+    auto mapPath = createMapCopyInTempDataPath("altar.map", "altar_without_info_file.map");
+
+    // Remove the info file
+    auto infoFilePath = mapPath;
+    infoFilePath.replace_extension("darkradiant");
+    fs::remove(infoFilePath);
+
+    // Listen for a warning message
+    WarningReceiver receiver;
+
+    GlobalCommandSystem().executeCommand("OpenMap", mapPath.string());
+
+    // The name of the .darkradiant file should appear in the message
+    auto infoFilename = infoFilePath.filename().string();
+    EXPECT_NE(receiver.getReceivedMessage().find(infoFilename), std::string::npos)
+        << "Didn't receive a warning about the missing .darkradiant file";
+}
+
 }
