@@ -21,7 +21,7 @@ TEST_F(ModelExportTest, ExportPatchMesh)
     // Load a map with a cylinder patch
     loadMap("modelexport_patch.map");
 
-    auto patchNode = algorithm::findFirstPatchWithMaterial(GlobalMapModule().findOrInsertWorldspawn(), 
+    auto patchNode = algorithm::findFirstPatchWithMaterial(GlobalMapModule().findOrInsertWorldspawn(),
         "textures/darkmod/wood/boards/ship_hull_medium");
     EXPECT_TRUE(patchNode);
 
@@ -49,7 +49,7 @@ TEST_F(ModelExportTest, ExportPatchMesh)
     GlobalCommandSystem().executeCommand("ExportSelectedAsModel", argList);
 
     auto model = GlobalModelCache().getModel(modRelativePath);
-    
+
     EXPECT_TRUE(model);
 
     // We expect exactly the same amount of vertices as in the patch mesh
@@ -83,13 +83,13 @@ TEST_F(ModelExportTest, ExportFolderNotExisting)
 
     constexpr const char* filename = "test.lwo";
     auto outputPath = outputFolder / filename;
-    
+
     EXPECT_NO_THROW(exporter->exportToPath(outputFolder.string(), filename)) << "Exporter failed to export to " << outputPath;
 
     EXPECT_TRUE(fs::exists(outputPath)) << "Exporter didn't create the file " << outputPath;
 }
 
-inline bool surfaceHasVertex(const model::IModelSurface& surface, const std::function<bool(const ArbitraryMeshVertex&)>& functor)
+inline bool surfaceHasVertex(const model::IModelSurface& surface, const std::function<bool(const MeshVertex&)>& functor)
 {
     for (int i = 0; i < surface.getNumVertices(); ++i)
     {
@@ -103,9 +103,9 @@ inline bool surfaceHasVertex(const model::IModelSurface& surface, const std::fun
 }
 
 const std::string CustomMaterialName = "custom_surface_name";
-const Vector3 VertexColour1(0.1, 0.2, 0.3);
-const Vector3 VertexColour2(0.4, 0.5, 0.6);
-const Vector3 VertexColour3(0.7, 0.8, 0.9);
+const Vector4 VertexColour1(0.1, 0.2, 0.3, 0.11);
+const Vector4 VertexColour2(0.4, 0.5, 0.6, 0.12);
+const Vector4 VertexColour3(0.7, 0.8, 0.9, 0.13);
 
 inline model::ModelNodePtr getChildModelNode(const scene::INodePtr& entity)
 {
@@ -160,15 +160,15 @@ inline void checkVertexColoursOfExportedModel(const model::IModelExporterPtr& ex
         EXPECT_EQ(surface.getNumVertices(), 3);
 
         // The three colours we exported need to be present in the mesh
-        EXPECT_TRUE(surfaceHasVertex(surface, [&](const ArbitraryMeshVertex& vertex)
+        EXPECT_TRUE(surfaceHasVertex(surface, [&](const MeshVertex& vertex)
         {
             return math::isNear(vertex.colour, VertexColour1, 0.01);
         }));
-        EXPECT_TRUE(surfaceHasVertex(surface, [&](const ArbitraryMeshVertex& vertex)
+        EXPECT_TRUE(surfaceHasVertex(surface, [&](const MeshVertex& vertex)
         {
             return math::isNear(vertex.colour, VertexColour2, 0.01);
         }));
-        EXPECT_TRUE(surfaceHasVertex(surface, [&](const ArbitraryMeshVertex& vertex)
+        EXPECT_TRUE(surfaceHasVertex(surface, [&](const MeshVertex& vertex)
         {
             return math::isNear(vertex.colour, VertexColour3, 0.01);
         }));
@@ -193,9 +193,9 @@ TEST_F(ModelExportTest, LwoVertexColoursAddedByPolygon)
 
     polys.emplace_back(model::ModelPolygon
     {
-        ArbitraryMeshVertex(Vertex3f(1,0,0), Normal3f(1,0,0), TexCoord2f(1,0), VertexColour1),
-        ArbitraryMeshVertex(Vertex3f(0,1,0), Normal3f(1,0,0), TexCoord2f(0,0), VertexColour2),
-        ArbitraryMeshVertex(Vertex3f(1,1,0), Normal3f(1,0,0), TexCoord2f(1,0), VertexColour3)
+        MeshVertex(Vertex3(1,0,0), Normal3(1,0,0), TexCoord2f(1,0), VertexColour1),
+        MeshVertex(Vertex3(0,1,0), Normal3(1,0,0), TexCoord2f(0,0), VertexColour2),
+        MeshVertex(Vertex3(1,1,0), Normal3(1,0,0), TexCoord2f(1,0), VertexColour3)
     });
 
     exporter->addPolygons(CustomMaterialName, polys, Matrix4::getIdentity());
@@ -207,7 +207,7 @@ class TestModelSurface :
     public model::IIndexedModelSurface
 {
 public:
-    std::vector<ArbitraryMeshVertex> vertices;
+    std::vector<MeshVertex> vertices;
     std::vector<unsigned int> indices;
 
     int getNumVertices() const override
@@ -220,11 +220,11 @@ public:
         return static_cast<int>(indices.size() / 3);
     }
 
-    const ArbitraryMeshVertex& getVertex(int vertexNum) const override
+    const MeshVertex& getVertex(int vertexNum) const override
     {
         return vertices[vertexNum];
     }
-    
+
     model::ModelPolygon getPolygon(int polygonNum) const override
     {
         return model::ModelPolygon
@@ -234,26 +234,39 @@ public:
             vertices[polygonNum * 3 + 2]
         };
     }
-    
+
     const std::string& getDefaultMaterial() const override
     {
         return CustomMaterialName;
     }
-    
+
     const std::string& getActiveMaterial() const override
     {
         return getDefaultMaterial();
     }
-    
-    const std::vector<ArbitraryMeshVertex>& getVertexArray() const override
+
+    const std::vector<MeshVertex>& getVertexArray() const override
     {
         return vertices;
     }
-    
+
     const std::vector<unsigned int>& getIndexArray() const override
     {
         return indices;
     }
+
+    const AABB& getSurfaceBounds() const override
+    {
+        static AABB aabb;
+        aabb = AABB();
+
+        for (const auto& vertex : vertices)
+        {
+            aabb.includePoint(vertex);
+        }
+
+        return aabb;
+    }
 };
 
 TEST_F(ModelExportTest, LwoVertexColoursAddedBySurface)
@@ -264,13 +277,13 @@ TEST_F(ModelExportTest, LwoVertexColoursAddedBySurface)
     // Create a few vertices with custom colours
     TestModelSurface surface;
 
-    surface.vertices.emplace_back(Vertex3f(1, 0, 0), Normal3f(1, 0, 0), TexCoord2f(1, 0), VertexColour1);
-    surface.vertices.emplace_back(Vertex3f(0, 1, 0), Normal3f(1, 0, 0), TexCoord2f(0, 0), VertexColour2);
-    surface.vertices.emplace_back(Vertex3f(1, 1, 0), Normal3f(1, 0, 0), TexCoord2f(1, 0), VertexColour3);
+    surface.vertices.emplace_back(Vertex3(1, 0, 0), Normal3(1, 0, 0), TexCoord2f(1, 0), VertexColour1);
+    surface.vertices.emplace_back(Vertex3(0, 1, 0), Normal3(1, 0, 0), TexCoord2f(0, 0), VertexColour2);
+    surface.vertices.emplace_back(Vertex3(1, 1, 0), Normal3(1, 0, 0), TexCoord2f(1, 0), VertexColour3);
     surface.indices.push_back(0);
     surface.indices.push_back(1);
     surface.indices.push_back(2);
-    
+
     exporter->addSurface(surface, Matrix4::getIdentity());
 
     checkVertexColoursOfExportedModel(exporter, _context.getTestProjectPath());
@@ -384,14 +397,15 @@ TEST_F(ModelExportTest, ExportedModelTriggersEntityRefresh)
 }
 
 // #5705: "Replace Selection with exported Model" should preserve spawnargs
+// #5858: "Replace Selection with exported Model" sets classname to "func_static".
 TEST_F(ModelExportTest, ExportedModelInheritsSpawnargs)
 {
     auto modelPath = "models/torch.lwo";
     auto exportedModelPath = "models/export_test.lwo";
     auto fullModelPath = _context.getTestProjectPath() + exportedModelPath;
 
-    // Create an entity referencing this new model
-    auto eclass = GlobalEntityClassManager().findClass("func_static");
+    // Create an entity referencing this new model, let's use a mover door
+    auto eclass = GlobalEntityClassManager().findClass("atdm:mover_door");
     auto entity = GlobalEntityModule().createEntity(eclass);
 
     scene::addNodeToContainer(entity, GlobalMapModule().getRoot());
@@ -450,10 +464,12 @@ TEST_F(ModelExportTest, ExportedModelInheritsSpawnargs)
     EXPECT_EQ(newEntity->getKeyValue("model"), exportedModelPath);
     EXPECT_EQ(newEntity->getKeyValue("dummy1"), "value1");
     EXPECT_EQ(newEntity->getKeyValue("dummy2"), "value2");
+    EXPECT_EQ(newEntity->getKeyValue("classname"), "atdm:mover_door");
+    EXPECT_EQ(newEntity->getEntityClass()->getName(), "atdm:mover_door");
 
     // This one should not have been inherited, it was belonging to the other entity
     EXPECT_EQ(newEntity->getKeyValue("henrys_key"), "");
-    
+
     fs::remove(fullModelPath);
 }
 
